unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, MSI_Common, MSI_Monitor, MSI_Defs,
  MSI_USB, MSI_Storage, MSI_Printers, MSI_Media, MSI_Network, MSI_Display,
  MSI_Memory, MSI_CPU, MSI_OS, MSI_Machine, MSI_Devices, MSI_SystemInfo, MSI_APM;

const
  WM_REFRESH = WM_USER+1;

type
  TForm1 = class(TForm)
    Tree: TTreeView;
    od: TOpenDialog;
    sd: TSaveDialog;
    ButtonPanel: TPanel;
    Button1: TButton;
    Button2: TButton;
    bSave: TButton;
    bLoad: TButton;
    Label1: TLabel;
    procedure bLoadClick(Sender: TObject);
    procedure bSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmRefresh(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure TreeCustomDrawItem(Sender: TCustomTreeView; Node: TTreeNode;
      State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure FormShow(Sender: TObject);
  private
    SI: TMiTeC_SystemInfo;
  protected
    procedure WMREFRESH(var AMsg: TMessage); message WM_REFRESH;
  public
    procedure RefreshData;
  end;

var
  Form1: TForm1;

implementation

uses StrUtils, TypInfo, MiTeC_USB, MiTeC_Routines, MiTeC_Windows, MiTeC_LSAAPI, MiTeC_StrUtils, MiTeC_Helpers, MiTeC_Firmware;

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  SI:=TMiTeC_SystemInfo.Create(Self);
  Caption:=SI._About;
  EnablePrivilege(SE_SYSTEM_ENVIRONMENT_NAME);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  PostMessage(Handle,WM_REFRESH,0,0);
end;

procedure TForm1.cmRefresh(Sender: TObject);
begin
  Screen.Cursor:=crHourglass;
  try
    Tree.Hide;
    ButtonPanel.Hide;
    Update;
    SI.OS.DetectUpdatesAndHotfixes:=False;
    SI.RefreshData(soOverview);
    RefreshData;
  finally
    ButtonPanel.Show;
    Screen.Cursor:=crDefault;
  end;
end;

procedure TForm1.bLoadClick(Sender: TObject);
var
  h: Boolean;
begin
  if not od.execute then
    Exit;
  h:=True;
  SI.LoadFromStorage(od.FileName,h);
  RefreshData;
end;

procedure TForm1.bSaveClick(Sender: TObject);
var
  h: Boolean;
begin
  if not sd.execute then
    Exit;
  h:=True;
  SI.SaveToStorage(sd.FileName,h);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.RefreshData;
var
  i,j: Integer;
  r,n,c: TTreeNode;
  s: string;
  di: TDiskInfo;
begin
  Screen.Cursor:=crHourglass;
  Tree.Items.BeginUpdate;
  Update;
  try
    Tree.Items.Clear;

    r:=Tree.Items.AddChild(nil,'Computer');
    n:=Tree.Items.AddChild(r,Format('%s - %s',[SI.Machine.MachineName,SI.Machine.User]));
    c:=Tree.Items.AddChild(n,Format('System: %s',[SI.Machine.Computer]));
    c:=Tree.Items.AddChild(n,Format('Model: %s',[SI.Machine.SMBIOS.SystemString]));

    r:=Tree.Items.AddChild(nil,'Operating system');
    s:=Format('%s',[SI.OS.TrueWindowsName]);
    n:=Tree.Items.AddChild(r,s);

    r:=Tree.Items.AddChild(nil,'Mainboard');
    n:=Tree.Items.AddChild(r,SI.Machine.GetMainboardName);
    Tree.Items.AddChild(n,'BIOS: '+SI.Machine.GetBIOSName+IfThen(SI.Machine.BIOS.UEFI,' (UEFI)',''));
    Tree.Items.AddChild(n,Format('Secure Boot: %s',[Copy(GetEnumName(TypeInfo(TUEFISecureBoot),Ord(SI.Machine.BIOS.SecureBoot)),7)]));
    Tree.Items.AddChild(n,Format('SMBIOS: %d.%d Rev. %d',[SI.Machine.SMBIOS.Reader.MajorVersion,SI.Machine.SMBIOS.Reader.MinorVersion,SI.Machine.SMBIOS.Reader.DMIRevision]));
    if SI.Machine.TPM.InterfaceVersion<>'' then
      Tree.Items.AddChild(n,Format('TPM: %s',[SI.Machine.TPM.InterfaceVersion])+IfThen(SI.Machine.TPM.Manufacturer<>'',Format(' - %s %s %s',[SI.Machine.TPM.Manufacturer,SI.Machine.TPM.Model,SI.Machine.TPM.Version]),''));

    r:=Tree.Items.AddChild(nil,'CPU');
    for i:=0 to SI.CPU.CPUPhysicalCount-1 do begin
      n:=Tree.Items.AddChild(r,Format('%s %s - %d MHz',[cVendorNames[SI.CPU.Vendor].Prefix,SI.CPU.CPUName,SI.CPU.Frequency]));
      with SI.CPU.Cache do
        c:=Tree.Items.AddChild(n,Format('Cache: %d x %d KB L1 + %d x %d KB L2 + %d x %d KB L3',[Level1.Code.SharedWays,Level1.Code.Size+Level1.Data.Size,Level2.SharedWays,Level2.Size,Level3.SharedWays,Level3.Size]));
      c:=Tree.Items.AddChild(n,Format('Core number: %d',[SI.CPU.CoreCount]));
      c:=Tree.Items.AddChild(n,Format('Thread number: %d',[SI.CPU.ThreadCount]));
      c:=Tree.Items.AddChild(n,Format('Instructions: %s',[SI.CPU.Features.Instructions]));
    end;

    r:=Tree.Items.AddChild(nil,'Memory');
    n:=Tree.Items.AddChild(r,Format('%s %s',[NormalizeDataValue(SI.Machine.SMBIOS.InstalledMemory),SI.Machine.SMBIOS.MemoryType]));
    for i:=0 to SI.Machine.SMBIOS.MemoryDeviceCount-1 do
      if SI.Machine.SMBIOS.MemoryDevice[i].Size>0 then
        c:=Tree.Items.AddChild(n,Format('%s: %s - %s',[SI.Machine.SMBIOS.MemoryDevice[i].BankLocator,NormalizeDataValue(SI.Machine.SMBIOS.MemoryDevice[i].SizeB),SI.Machine.SMBIOS.MemoryDevice[i].Manufacturer]));

    r:=Tree.Items.AddChild(nil,'Storage');
    for i:=0 to SI.Storage.PhysicalCount-1 do begin
      n:=Tree.Items.AddChild(r,Format('%s - %d MB',[SI.Storage.Physical[i].Model,SI.Storage.Physical[i].Size shr 20]));
      for j:=0 to SI.Storage.LogicalCount-1 do
        if SI.Storage.Logical[j].PhysicalIndex=i then begin
          di:=GetDiskInfo(SI.Storage.Logical[j].Drive+':');
          c:=Tree.Items.AddChild(n,Format('%s: [%s] - %s (%d MB)',[SI.Storage.Logical[j].Drive,di.Volumelabel,di.FileSystem,di.Capacity shr 20]));
        end;
    end;

    r:=Tree.Items.AddChild(nil,'USB');
    for i:=0 to SI.USB.USBNodeCount-1 do begin
      s:='';
      if (SI.USB.USBNodes[i].USBDevice.ConnectionStatus=1) and
         not(SI.USB.USBNodes[i].USBClass in [usbHostController,usbHub,usbExternalHub]) then begin
         if SI.USB.USBNodes[i].USBDevice.Product<>'' then
           s:=s+Format('Port[%d]: %s',[SI.USB.USBNodes[i].USBDevice.Port,SI.USB.USBNodes[i].USBDevice.Product])
         else
           s:=s+Format('Port[%d]: %s',[SI.USB.USBNodes[i].USBDevice.Port,SI.USB.USBNodes[i].USBDevice.USBClassname]);
        s:=s+Format(' (%d.%d)',[SI.USB.USBNodes[i].USBDevice.MajorVersion,SI.USB.USBNodes[i].USBDevice.MinorVersion]);
        n:=Tree.Items.AddChild(r,s);
      end;
    end;

    r:=Tree.Items.AddChild(nil,'Graphics');
    for i:=0 to SI.Display.AdapterCount-1 do begin
      n:=Tree.Items.AddChild(r,Format('%s - %d MB',[SI.Display.Adapter[i].Name,SI.Display.Adapter[i].Memory shr 20]));
    end;


    r:=Tree.Items.AddChild(nil,'Monitor');
    for i:=0 to SI.Monitor.Count-1 do begin
      s:=SI.Monitor.Monitors[i].EDIDRecord.Name;
      if s='' then
        s:=SI.Monitor.Monitors[i].DeviceDescription;
      n:=Tree.Items.AddChild(r,Format('%s %1.1f" (%d x %d x %dbits - %d dpi)',[s,SI.Monitor.Monitors[i].EDIDRecord.Diagonal,RectWidth(SI.Monitor.Monitors[i].Bounds),RectHeight(SI.Monitor.Monitors[i].Bounds),SI.Monitor.Monitors[i].ColorDepth,SI.Monitor.Monitors[i].DPI]));
    end;

    r:=Tree.Items.AddChild(nil,'Audio');
    if SI.Media.SoundCardName<>'' then
      n:=Tree.Items.AddChild(r,SI.Media.SoundCardName)
    else
      with SI.Media do
        if (WAVEOut.Count>0) then
          n:=Tree.Items.AddChild(r,WaveOut[0]);

    r:=Tree.Items.AddChild(nil,'Network');
    if SI.Network.TCPIP.AdapterCount>0 then
      for i:=0 to SI.Network.TCPIP.AdapterCount-1 do begin
        n:=Tree.Items.AddChild(r,Format('%s - %s (%s)',[SI.Network.TCPIP.Adapter[i].Name,SI.Network.TCPIP.Adapter[i].Address,SI.Network.TCPIP.Adapter[i].IPAddress.CommaText]));
      end
    else
      for i:=0 to SI.Network.PhysicalAdapters.Count-1 do begin
        n:=Tree.Items.AddChild(r,Format('%s - %s (%s)',[SI.Network.PhysicalAdapters[i],SI.Network.MACAddresses[i],SI.Network.IPAddresses[i]]));
      end;

    r:=Tree.Items.AddChild(nil,'Printers');
    for i:=0 to SI.Printers.PrinterCount-1 do begin
      n:=Tree.Items.AddChild(r,Format('%s - %s',[SI.Printers.Driver[i],SI.Printers.Port[i]]));
    end;

    Tree.FullExpand;

    Tree.Items.GetFirstNode.MakeVisible;
  finally
    Tree.Items.EndUpdate;
    Tree.Show;
    Screen.Cursor:=crdefault;
  end;
end;

procedure TForm1.TreeCustomDrawItem(Sender: TCustomTreeView;
  Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
begin
  Sender.Canvas.Font.Style:=[];
  if Assigned(Node) then begin
    case Node.Level of
      0: Sender.Canvas.Font.Style:=[fsBold,fsUnderline];
      2: Sender.Canvas.Font.Color:=clGray;
    end;
  end;
end;

procedure TForm1.WMREFRESH(var AMsg: TMessage);
begin
  cmRefresh(nil);
end;

end.
