{.$DEFINE DLM}

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, StdCtrls, Menus,
  MSI_SysProcMon, MSI_SvcListMon, MiTeC_Routines;

type
  TKillThread = class(TThread)
  private
    FPID: Cardinal;
    FTimeout: integer;
    FKS: TTerminateStatus;
  protected
    procedure Sync;
    procedure Execute; override;
  public
    constructor Create(APID: Cardinal; ATimeout: Cardinal = INFINITE); reintroduce;
  end;

  Twnd_Main = class(TForm)
    PrcList: TListView;
    pc: TPageControl;
    tsPrc: TTabSheet;
    Button2: TButton;
    tsSvc: TTabSheet;
    MainMenu: TMainMenu;
    File1: TMenuItem;
    mmRefresh: TMenuItem;
    N1: TMenuItem;
    mmExit: TMenuItem;
    Bevel1: TBevel;
    mmProcess: TMenuItem;
    mmPrcDetails: TMenuItem;
    mmPrcProps: TMenuItem;
    N3: TMenuItem;
    mmPrcKill: TMenuItem;
    mmDriver: TMenuItem;
    mmDrvProps: TMenuItem;
    mmService: TMenuItem;
    mmSvcProps: TMenuItem;
    mSvcDetails: TMenuItem;
    N4: TMenuItem;
    mmSvcStart: TMenuItem;
    mmSvcStop: TMenuItem;
    mmSvcPause: TMenuItem;
    mmSvcResume: TMenuItem;
    mmAbout: TMenuItem;
    SvcList: TListView;
    Button5: TButton;
    cbSvc: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure ListCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure ListColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListAdvancedCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
      var DefaultDraw: Boolean);
    procedure ListAdvancedCustomDrawSubItem(Sender: TCustomListView;
      Item: TListItem; SubItem: Integer; State: TCustomDrawState;
      Stage: TCustomDrawStage; var DefaultDraw: Boolean);
    procedure cmRefresh(Sender: TObject);
    procedure cmExit(Sender: TObject);
    procedure cmPrcDetails(Sender: TObject);
    procedure pcChange(Sender: TObject);
    procedure mmAboutClick(Sender: TObject);
    procedure cmPrcKill(Sender: TObject);
    procedure cmSvcDetails(Sender: TObject);
    procedure cmSvcStart(Sender: TObject);
    procedure cmSvcStop(Sender: TObject);
    procedure cmSvcPause(Sender: TObject);
    procedure cmSvcResume(Sender: TObject);
    procedure SysProcMonThreadInterval(Sender: TSysProcMonThread);
    procedure SvcListMonThreadInterval(Sender: TSvcListMonThread);
    procedure cbSvcChange(Sender: TObject);
  private
    FKT: TKillThread;
    sl: TStringList;
  public
    procedure RefreshProcList(Sender: TSysProcMonThread);
    procedure RefreshSvcList(Sender: TSvcListMonThread);
  end;

var
  wnd_Main: Twnd_Main;
  SPM: TSysProcMonThread;
  SLM: TSvcListMonThread;

implementation

uses ShellAPI, MiTeC_CtrlRtns, MiTeC_Datetime, MiTeC_Dialogs, PrcDetails,
  MiTeC_CommonDefs, MiTeC_AdvAPI, SvcDetails, WinSvc, MSI_Defs, MiTeC_Windows, MiTeC_LSAAPI;

{$R *.dfm}

procedure Twnd_Main.FormCreate(Sender: TObject);
begin
  EnablePrivilege(SE_DEBUG_NAME);
  EnablePrivilege(SE_SECURITY_NAME);
  EnablePrivilege(SE_SYSTEM_ENVIRONMENT_NAME);
  sl:=TStringList.Create;
  SPM:=TSysProcMonThread.Create;
  SPM.OnInterval:=SysProcMonThreadInterval;
  SPM.AutoSuspend:=True;
  SLM:=TSvcListMonThread.Create;
  SLM.Types:=stAll;
  SLM.AutoSuspend:=True;
  SLM.OnInterval:=SvcListMonThreadInterval;
  {$IFDEF DLM}
  DLM:=TSysModListMonThread.Create;
  DLM.OnInterval:=SysModListMonThreadInterval;
  tsDrv.TabVisible:=True;
  {$ENDIF}
  cmRefresh(nil);
  pc.ActivePageIndex:=0;
  Screen.Cursor:=crHourGlass;
end;

procedure Twnd_Main.RefreshProcList(Sender: TSysProcMonThread);
var
  i: Integer;
  r: TProcessRecord;
begin
  PrcList.Items.BeginUpdate;
  try
    PrcList.Items.Clear;
    for i:=0 to Sender.RecordCount-1 do begin
      Sender.GetRecord(i,r);
      with PrcList.Items.Add do begin
        Caption:=r.Name;
        if Is64 and (r.Platform<>64) then
          Caption:=Caption+'*32';
        SubItems.Add(Format('%d',[r.PID]));
        SubItems.Add(FormatTicks(r.UserTime+r.KernelTime,False));
        SubItems.Add(Format('%d KB',[r.VMCounters.WorkingSetSize div 1024]));
        SubItems.Add(Format('%s',[r.CommandLine]));
      end;
    end;
    if PrcList.Tag<>0 then
      PrcList.AlphaSort;
  finally
    PrcList.Items.EndUpdate;
  end;
end;

procedure Twnd_Main.RefreshSvcList(Sender: TSvcListMonThread);
var
  i: Integer;
  r: TServiceRecord;
begin
  SvcList.Items.BeginUpdate;
  try
    SvcList.Items.Clear;
    for i:=0 to Sender.RecordCount-1 do begin
      Sender.GetRecord(i,r);
      with SvcList.Items.Add do begin
        Caption:=r.DisplayName;
        if r.Typ=svcUnknown then
          SubItems.Add(Format('%s (%d)',[cSvcType[r.Typ],r._Typ]))
        else
          SubItems.Add(cSvcType[r.Typ]);
        SubItems.Add(cSvcStatus[r.Status]);
        SubItems.Add(cSvcStartup[r.StartUp]);
        SubItems.Add(r.ObjectName);
        SubItems.Add(r.Name);
      end;
    end;
    if SvcList.Tag<>0 then
      SvcList.AlphaSort;
  finally
    SvcList.Items.EndUpdate;
  end;
end;

procedure Twnd_Main.SvcListMonThreadInterval(Sender: TSvcListMonThread);
begin
  tsSvc.Caption:=Format('Services (%d)',[Sender.RecordCount]);
  Update;
  Screen.Cursor:=crHourGlass;
  try
    RefreshSvcList(Sender);
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure Twnd_Main.ListCompare(Sender: TObject; Item1, Item2: TListItem;
  Data: Integer; var Compare: Integer);
begin
  Compare:=ListView_CustomSort(Item1,Item2,abs(TListView(Sender).Tag)-1);
  if TListView(Sender).Tag<0 then
    Compare:=-Compare;
end;

procedure Twnd_Main.ListColumnClick(Sender: TObject; Column: TListColumn);
begin
  TListView(Sender).SortType:=stNone;
  if Column.Index+1<>abs(TListView(Sender).Tag) then
    TListView(Sender).Tag:=Column.Index+1
  else
    TListView(Sender).Tag:=-TListView(Sender).Tag;
  TListView(Sender).SortType:=stText;
end;

procedure Twnd_Main.ListAdvancedCustomDrawItem(Sender: TCustomListView;
  Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
  var DefaultDraw: Boolean);
begin
  if Abs(TListView(Sender).Tag)=1 then
    Sender.Canvas.Brush.Color:=clInfoBk
  else
    Sender.Canvas.Brush.Color:=clWhite
end;

procedure Twnd_Main.ListAdvancedCustomDrawSubItem(Sender: TCustomListView;
  Item: TListItem; SubItem: Integer; State: TCustomDrawState;
  Stage: TCustomDrawStage; var DefaultDraw: Boolean);
begin
  if Abs(TListView(Sender).Tag)=SubItem+1 then
    Sender.Canvas.Brush.Color:=clInfoBk
  else
    Sender.Canvas.Brush.Color:=clWhite
end;

procedure Twnd_Main.cmRefresh(Sender: TObject);
begin
  SPM.Suspended:=False;
  SPM.OnInterval:=SysProcMonThreadInterval;
  SLM.Suspended:=False;
  {$IFDEF DLM}
  DLM.Suspended:=False;
  {$ENDIF}
end;

procedure Twnd_Main.cmExit(Sender: TObject);
begin
  Close;
end;

procedure Twnd_Main.cmPrcDetails(Sender: TObject);
var
  r: TProcessRecord;
begin
  if not Assigned(PrcList.Selected) then
    Exit;
  Screen.Cursor:=crHourglass;
  Sleep(1000);
  SPM.SetWatchedProcess(StrToInt(PrcList.Selected.SubItems[0]),True);
  try
    SPM.GetRecordByPID(StrToInt(PrcList.Selected.SubItems[0]),r);
    SPM.GetChildProcesses(StrToInt(PrcList.Selected.SubItems[0]),sl);
    ShowPrcDetails(r,sl);
  finally
    SPM.SetWatchedProcess(r.PID,False);
  end;
end;

procedure Twnd_Main.pcChange(Sender: TObject);
begin
  mmProcess.Visible:=pc.ActivePage.pageIndex=0;
  mmService.Visible:=pc.ActivePage.pageIndex=1;
  mmDriver.Visible:=pc.ActivePage.pageIndex=2;
end;

procedure Twnd_Main.SysProcMonThreadInterval(Sender: TSysProcMonThread);
begin
  tsPrc.Caption:=Format('Processes (%d)',[Sender.RecordCount]);
  Update;
  Screen.Cursor:=crHourGlass;
  try
    RefreshProcList(Sender);
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure Twnd_Main.mmAboutClick(Sender: TObject);
begin
  ShellAbout(Handle,PChar(Application.Title+' '+cVersion),PChar(Format('%s (%s)',[cCopyright,cWWW]))  ,Application.Icon.Handle);
end;

procedure Twnd_Main.cmPrcKill(Sender: TObject);
var
  pid: Cardinal;
begin
  if not Assigned(PrcList.Selected) then
    Exit;

  pid:=StrToInt(PrcList.Selected.SubItems[0]);
  if (pid>9) then
    if WarnYesNo('WARNING: Terminating a process can cause undesired '#13#10+
                  'results including loss of data and system instability. The '#13#10+
                  'process will not be given the chance to save its state or '#13#10+
                  'data before it is terminated. Are you sure you want to '#13#10+
                  'terminate the process?') then begin
      FKT:=TKillThread.Create(pid,30000);
    end;
end;

procedure Twnd_Main.cmSvcDetails(Sender: TObject);
var
  r: TServiceRecord;
begin
  if not Assigned(SvcList.Selected) then
    Exit;

  SLM.GetRecordByName(SvcList.Selected.SubItems[SvcList.Selected.SubItems.Count-1],r);
  ShowSvcDetails(r);
end;

procedure Twnd_Main.cmSvcStart(Sender: TObject);
var
  r: TServiceRecord;
  s: string;
begin
  if not Assigned(SvcList.Selected) then
    Exit;

  SLM.GetRecordByName(SvcList.Selected.SubItems[SvcList.Selected.SubItems.Count-1],r);
  if (r.Status=SERVICE_STOPPED) and
     YesNo(Format('Start %s service?',[r.DisplayName])) and
     InputQuery(r.DisplayName,'Parameters',s) then begin
    if ServiceStart(r.Name,s) then
      SLM.Suspended:=False
    else
      Warn('Cannot start service.');
  end;
end;

procedure Twnd_Main.cmSvcStop(Sender: TObject);
var
  r: TServiceRecord;
begin
  if not Assigned(SvcList.Selected) then
    Exit;

  SLM.GetRecordByName(SvcList.Selected.SubItems[SvcList.Selected.SubItems.Count-1],r);

  if (r.ControlsAccepted and SERVICE_ACCEPT_STOP>0) and (r.Status in [SERVICE_RUNNING,SERVICE_PAUSED]) and YesNo(Format('Stop %s service?',[r.DisplayName])) then begin
    ServiceGetDependants('',r.Name,True,False,sl);
    if ServiceStop(r.Name,sl,True) then
      SLM.Suspended:=False
    else
      Warn('Cannot stop service.');
  end;
end;

procedure Twnd_Main.cmSvcPause(Sender: TObject);
var
  r: TServiceRecord;
begin
  if not Assigned(SvcList.Selected) then
    Exit;

  SLM.GetRecordByName(SvcList.Selected.SubItems[SvcList.Selected.SubItems.Count-1],r);
  if (r.Status=SERVICE_RUNNING) and YesNo(Format('Pause %s service?',[r.DisplayName])) then begin
    if ServicePause(r.Name) then
      SLM.Suspended:=False
    else
      Warn('Cannot pause service.');
  end;
end;

procedure Twnd_Main.cmSvcResume(Sender: TObject);
var
  r: TServiceRecord;
begin
  if not Assigned(SvcList.Selected) then
    Exit;

  SLM.GetRecordByName(SvcList.Selected.SubItems[SvcList.Selected.SubItems.Count-1],r);
  if (r.Status=SERVICE_PAUSED) and YesNo(Format('Resume %s service?',[r.DisplayName])) then begin
    if ServiceContinue(r.Name) then
      SLM.Suspended:=False
    else
      Warn('Cannot resume service.');
  end;
end;

procedure Twnd_Main.cbSvcChange(Sender: TObject);
begin
  case cbSvc.ItemIndex of
    0: SLM.Types:=stServices;
    1: SLM.Types:=stDrivers;
    2: SLM.Types:=stAll;
  end;
  SLM.Suspended:=False;
end;

{ TKillThread }

constructor TKillThread.Create(APID: Cardinal; ATimeout: cardinal = INFINITE);
begin
  FPID:=APID;
  FTimeout:=ATimeout;
  FreeOnTerminate:=True;
  inherited Create(False);
end;

procedure TKillThread.Execute;
begin
  FKS:=TSysProcMonThread.KillProcess(FPID,FTimeout);
  Synchronize(Sync);
end;

procedure TKillThread.Sync;
begin
  if FKS=tsError then
    Warn('Cannot terminate this process.')
  else
    SPM.Suspended:=False;
end;

end.
