unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, MiTeC_EventLogNT, MSI_EventLog, ComCtrls, StdCtrls, ImgList, ExtCtrls;

type
  TForm1 = class(TForm)
    lv: TListView;
    EventImages: TImageList;
    Memo: TMemo;
    Panel1: TPanel;
    bSave: TButton;
    bLoad: TButton;
    od: TOpenDialog;
    sd: TSaveDialog;
    bAction: TButton;
    bBackup: TButton;
    sdEvt: TSaveDialog;
    pc: TPageControl;
    tsDesc: TTabSheet;
    tsValues: TTabSheet;
    Memo1: TMemo;
    Panel2: TPanel;
    bOpenFile: TButton;
    bOpenCont: TButton;
    cbCont: TComboBox;
    cbxEM: TCheckBox;
    odEvt: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure bOpenClick(Sender: TObject);
    procedure lvSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure bSaveClick(Sender: TObject);
    procedure bLoadClick(Sender: TObject);
    procedure cmRefresh(Sender: TObject);
    procedure ELReadEventLog(Sender: TObject; const ARecord: TLogRecord; var Cancel: Boolean);
    procedure cmCancel(Sender: TObject);
    procedure bBackupClick(Sender: TObject);
    procedure lvData(Sender: TObject; Item: TListItem);
  private
    FCancel: boolean;
    EL: TMiTeC_EventLog;
    et: comp;
  public
  end;

var
  Form1: TForm1;

implementation

uses MiTeC_Windows, MiTeC_Routines;

{$R *.dfm}

procedure TForm1.bLoadClick(Sender: TObject);
var
  i: Integer;
  h: Boolean;
begin
  if not od.Execute then
    Exit;
  cbCont.Items.Clear;
  lv.Items.Clear;
  Memo.Lines.Clear;
  Memo1.Lines.Clear;
  h:=True;
  EL.LoadFromStorage(od.FileName,h);
  for i:=0 to EL.ContainerCount-1 do
    cbCont.Items.Add(EL.Containers[i].Name);
end;

procedure TForm1.bSaveClick(Sender: TObject);
var
  i: Integer;
  h: Boolean;
begin
  sd.FileName:=Format('%s.sif',[Machinename]);
  if not sd.Execute then
    Exit;
  Screen.Cursor:=crHourglass;
  try
    EL.SourceFilter:='';
    h:=True;
    for i:=0 to EL.ContainerCount-1 do
      try
        EL.SourceName:=EL.Containers[i].Name;
        EL.SaveToStorage(sd.FileName,h);
      except
      end;
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure TForm1.bBackupClick(Sender: TObject);
begin
  if cbCont.ItemIndex=-1 then
    Exit;
  sdEvt.FileName:=cbCont.Text+'.evtx';
  if not sdEvt.Execute then
    Exit;
  Screen.Cursor:=crHourglass;
  try
    EL.Backup(cbCont.Text,sdEvt.FileName);
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure TForm1.cmCancel(Sender: TObject);
begin
  FCancel:=True;
end;

procedure TForm1.cmRefresh(Sender: TObject);
var
  i: Integer;
begin
  cbCont.Items.Clear;
  lv.Items.Clear;
  Memo.Lines.Clear;
  Memo1.Lines.Clear;
  EL.RefreshData;
  for i:=0 to EL.ContainerCount-1 do
    cbCont.Items.Add(EL.Containers[i].Name);
end;

procedure TForm1.bOpenClick(Sender: TObject);
begin
  Memo.Lines.Clear;
  Memo1.Lines.Clear;
  tsDesc.Caption:=Format('Description (%d)',[Memo.Lines.Count]);
  tsValues.Caption:=Format('Values (%d)',[Memo1.Lines.Count]);
  if ((Sender=bOpenCont) and (cbCont.ItemIndex=-1)) or ((Sender=bOpenFile) and not odEvt.Execute) then
    Exit;

  bAction.Caption:='Cancel';
  bAction.OnClick:=cmCancel;
  bLoad.Enabled:=False;
  bSave.Enabled:=False;
  bBackup.Enabled:=False;
  bOpenFile.Enabled:=False;
  bOpenCont.Enabled:=False;
  cbCont.Enabled:=False;
  lv.Enabled:=False;
  pc.Enabled:=False;
  cbxEM.Enabled:=False;
  FCancel:=False;
  Screen.Cursor:=crHourglass;
  try
    et:=GetTickCountSafe;
    EL.ClearRecords;
    EL.ExpandMessages:=cbxEM.Checked;
    if (Sender=bOpenCont) then
      EL.SourceName:=cbCont.Text
    else
      EL.SourceName:=odEvt.FileName;
    FCancel:=False;
    Caption:=Format('EventLog Viewer - %d records / %1.2f s',[EL.RecordCount,(GetTickCountSafe-et)/1000]);
    lv.Items.Count:=EL.RecordCount;
  finally
    bAction.Caption:='Refresh';
    bAction.OnClick:=cmRefresh;
    bLoad.Enabled:=True;
    bSave.Enabled:=True;
    bBackup.Enabled:=True;
    cbCont.Enabled:=True;
    bOpenFile.Enabled:=True;
    bOpenCont.Enabled:=True;
    lv.Enabled:=True;
    pc.Enabled:=True;
    cbxEM.Enabled:=True;
    Screen.Cursor:=crDefault;
  end;
  lv.SetFocus;
end;

procedure TForm1.ELReadEventLog(Sender: TObject; const ARecord: TLogRecord; var Cancel: Boolean);
begin
  Cancel:=FCancel;
  if EL.RecordCount mod 1000=0 then begin
    Caption:=Format('EventLog Viewer - %d records',[EL.RecordCount]);
    Application.ProcessMessages;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  EL:=TMiTeC_EventLog.Create(nil);
  EL.OnReadEventLog:=ELReadEventLog;
  EL.ForceOldAPI:=False;
  EL.LookupSID:=True;
  for i:=0 to EL.ContainerCount-1 do
    cbCont.Items.Add(EL.Containers[i].Name);
end;

procedure TForm1.lvData(Sender: TObject; Item: TListItem);
var
  r: TLogRecord;
begin
  r:=EL.Records[Item.Index];
  Item.Caption:=DatetimeToStr(r.DateTime);
  Item.SubItems.Add(r.Source);
  Item.SubItems.Add(IntToStr(r.EventID));
  if EL.ExpandMessages then
    Item.SubItems.Add(r.Category)
  else
    Item.SubItems.Add('Check ''Expand messages'' to see data');
  Item.SubItems.Add(r.Computer);
  Item.ImageIndex:=Integer(r.EventType);
end;

procedure TForm1.lvSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  if not Assigned(Item) then
    Exit;
  if EL.ExpandMessages then begin
    Memo.Lines.Text:=EL.Records[Item.Index].Description;
    tsDesc.Caption:=Format('Description (%d)',[Memo.Lines.Count]);
  end else
    Memo.Lines.Text:='Check ''Expand messages'' to see data';
  Memo1.Lines.Text:=EL.Records[Item.Index].Values;
  tsValues.Caption:=Format('Values (%d)',[Memo1.Lines.Count]);
end;

end.
