unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Menus,  MSI_Common, MSI_Defs;

type
  Twnd_cputest_Main = class(TForm)
    Box: TMemo;
    Panel2: TPanel;
    Button1: TButton;
    Panel3: TPanel;
    Button2: TButton;
    cbxSMBIOS: TCheckBox;
    cbxWMI: TCheckBox;
    cbxSave: TCheckBox;
    cbxXML: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
  public
    procedure DetectCPU;
  end;

var
  wnd_cputest_Main: Twnd_cputest_Main;

implementation

uses Registry, MSI_CPU, MSI_SMBIOS, MiTeC_WbemScripting_TLB, MiTeC_WMI, MSI_XML_Reports,
  MiTeC_Routines, MiTeC_Windows, MiTeC_NativeAPI, MiTeC_RegUtils;

{$R *.DFM}

function RoundFrequency(const Frequency: Integer): Integer;
const
  NF: array [0..9] of Integer = (0, 20, 33, 50, 60, 66, 80, 90, 100, 133);
var
  Freq, RF: Integer;
  i: Byte;
  Hi, Lo: Byte;
begin
  RF:=0;
  Freq:=Frequency mod 100;
  for i:=0 to High(NF) do begin
    if Freq<NF[i] then begin
      Hi:=i;
      Lo:=i-1;
      if (NF[Hi]-Freq)>(Freq-NF[Lo]) then
        RF:=NF[Lo]-Freq
      else
        RF:=NF[Hi]-Freq;
      Break;
    end;
  end;
  Result:=Frequency+RF;
end;

function GetSystemProductName: string;
var
  f,m: string;
begin
  m:='';
  f:='';
  Result:='';
  with OpenRegistryReadOnly do
    try
      RootKey:=HKEY_LOCAL_MACHINE;
      if OpenKey('HARDWARE\DESCRIPTION\System\BIOS',False) then begin
        if ValueExists('SystemProductName') then
          Result:=ReadString('SystemProductName');
        if ValueExists('SystemFamily') then
          f:=ReadString('SystemFamily');
        if ValueExists('SystemManufacturer') then
          m:=ReadString('SystemManufacturer');
        if (Pos(f,Result)=0) and (Pos('=',f)=0) then
          Result:=f+' '+Result;
        if Pos(m,Result)=0 then
          Result:=m+' '+Result;
        CloseKey;
      end;
    finally
      Free;
    end;
end;

function GetBaseBoardName: string;
var
  s: string;
begin
  s:='';
  Result:='';
  with OpenRegistryReadOnly do
    try
      RootKey:=HKEY_LOCAL_MACHINE;
      if OpenKey('HARDWARE\DESCRIPTION\System\BIOS',False) then begin
        if ValueExists('BaseBoardManufacturer') then
          Result:=Result+ReadString('BaseBoardManufacturer');
        if ValueExists('BaseBoardProduct') then
          Result:=Result+' '+ReadString('BaseBoardProduct');
        if ValueExists('BaseBoardVersion') then begin
          s:=ReadString('BaseBoardVersion');
          if not SameText(s,'N/A') and (s<>'') then
            Result:=Result+' '+s;
        end;
        Result:=Trim(Result);
        CloseKey;
      end;
    finally
      Free;
    end;
end;
function GetBIOS: string;
begin
  Result:='';
  with OpenRegistryReadOnly do
    try
      RootKey:=HKEY_LOCAL_MACHINE;
      if OpenKey('HARDWARE\DESCRIPTION\System\BIOS',False) then begin
        if ValueExists('BIOSVendor') then
          Result:=Result+ReadString('BIOSVendor');
        if ValueExists('BIOSVersion') then
          Result:=Result+' '+ReadString('BIOSVersion');
        if ValueExists('BIOSReleaseDate') then
          Result:=Result+' '+ReadString('BIOSReleaseDate');
        Result:=Trim(Result);
        CloseKey;
      end;
    finally
      Free;
    end;
end;

procedure Twnd_cputest_Main.Button1Click(Sender: TObject);
begin
  Close;
end;

procedure Twnd_cputest_Main.DetectCPU;
var
   i,j,m: Integer;
   s,f,fn: string;
   wmiServices: ISWbemServices;
   wmi: TInstances;
   CPUID: TCPUIDResult;
   CPU: TMiTeC_CPU;
   sl: TStringList;
   buf1,slpi: PSystemLogicalProcessorInformation;
   buf2,slpiex: PSystemLogicalProcessorInformationEx;
   n,c: Cardinal;
   h: Boolean;
begin
  with Box, Lines do begin
    Add('[CPU Test]');
    Add(Format('Version = %s',[cVersion]));
    {$IFDEF WIN32}
    Add('Target platform: 32-bit');
    {$ENDIF}
    {$IFDEF WIN64}
    Add('Target platform: 64-bit');
    {$ENDIF}
    Add('OS: '+FormOSName);
    Add('Machine: '+MachineName);
    Add('Product: '+GetSystemProductName);
    Add('Baseboard: '+GetBaseBoardName);
    Add('BIOS: '+GetBIOS);
    Add('');
    Add('[System]');
    Add(Format('Number of Logical Processors = %d',[SystemInfo.dwNumberOfProcessors]));
    Add(Format('Active Processor Mask = %d bits set',[CountSetBits(SystemInfo.dwActiveProcessorMask)]));
    Add(Format('Processor Revision = Model %d Stepping %d (0x%4.4x)',[Hi(SystemInfo.wProcessorRevision),Lo(SystemInfo.wProcessorRevision),SystemInfo.wProcessorRevision]));
    Add('');
    Add('[General]');
    try
      CPU:=TMiTeC_CPU.Create(Self);
      with CPU do
        try
          Refreshdata;

          Add(Format('Number of Sockets = %d',[SocketCount]));
          Add(Format('Number of Physical Processors = %d',[CPUPhysicalCount]));

          Add(Format('Total Number of Cores = %d',[CoreCount]));
          Add(Format('Total Number of Threads = %d',[ThreadCount]));

          Add(Format('Number of Cores per Package = %d',[CorePerPackage]));
          Add(Format('Number of Threads per Core = %d',[LogicalPerCore]));
          Add(Format('Number of Threads per Package = %d',[LogicalPerPackage]));

          Add(Format('Max number of Threads per Core = %d',[MaxLogicalPerCore]));
          Add(Format('Max number of Cores per Package = %d',[MaxCorePerPackage]));
          Add(Format('Max number of Threads per Package = %d',[MaxLogicalPerPackage]));
          Add('');
          c:=CPUPhysicalCount;

          Add('[Processor]');
          case Architecture of
            PROCESSOR_ARCHITECTURE_AMD64: Add('Architecture = x64 (AMD or Intel)');
            PROCESSOR_ARCHITECTURE_IA32_ON_WIN64: Add('Architecture = WOW64');
            PROCESSOR_ARCHITECTURE_IA64: Add('Architecture = Intel Itanium Processor Family (IPF)');
            PROCESSOR_ARCHITECTURE_INTEL: Add('Architecture = x86');
            else Add(Format('Architecture = %d',[Architecture]));
          end;
          Add(Format('Socket Designation = %s',[SocketDesignation]));
          Add(Format('Type = %d',[SystemInfo.dwProcessorType]));
          Add(Format('Level = %d',[SystemInfo.wProcessorLevel]));
          Add(Format('LogicalPerPackage = %d',[LogicalPerPackage]));
          Add(Format('CorePerPackage = %d',[CorePerPackage]));
          Add(Format('LogicalPerCore = %d',[LogicalPerCore]));
          Add(Format('Vendor = %s',[cVendorNames[Vendor].Name]));
          Add(Format('CPUName = %s',[CPUName]));
          Add(Format('GenericName = %s',[GenericName]));
          Add(Format('MarketingName = %s',[MarketingName]));
          Add(Format('CodeName = %s',[CodeName]));
          Add(Format('Revision = %s',[Revision]));
          Add(Format('MicroCode Revision = %s',[MicroCodeRevision]));
          Add(Format('Technology = %s',[Technology]));
          Add(Format('Brand = %d',[Brand]));
          Add(Format('Type = %d',[CPUType]));
          Add(Format('Family = %d (0x%x)',[Family,Family]));
          Add(Format('Model = %d (0x%x)',[Model,Model]));
          Add(Format('Stepping = %d (0x%x)',[Stepping,Stepping]));
          Add(Format('FamilyEx = %d (0x%x)',[FamilyEx,FamilyEx]));
          Add(Format('ModelEx = %d (0x%x)',[ModelEx,ModelEx]));
          Add(Format('SteppingEx = %d (0x%x)',[SteppingEx,SteppingEx]));
          Add(Format('Frequency = %d MHz',[Frequency]));
          Add(Format('Serial Number = %s',[SerialNumber]));
          f:=GenericName;

          Add('');
          Add('[Cache]');
          with Cache.Level1.Code do
            if Typ>0 then
              Add(Format('%d x %s',[SharedWays,Descriptor]));
          with Cache.Level1.Data do
            if Typ>0 then
              Add(Format('%d x %s',[SharedWays,Descriptor]));
          with Cache.Level1.Unified do
            if Typ>0 then
              Add(Format('%d x %s',[SharedWays,Descriptor]));
          with Cache.Level2 do begin
            if Typ>0 then
              Add(Format('%d x %s',[SharedWays,Descriptor]));
          end;
          with Cache.Level3 do begin
            if Typ>0 then
              Add(Format('%d x %s',[SharedWays,Descriptor]));
          end;
          with Cache.Trace do begin
            if Typ>0 then
              Add(Descriptor);
          end;

          s:='';
          if Vendor=cvIntel then begin
            with Cache.Level1.Code do
              for j:=1 to 16 do
                s:=s+Format('%2.2x ',[Descriptors[j]]);
            Add(Format('Level1 Descriptors = %s',[s]));
          end;
          s:='';
          with Cache.Level2 do begin
            if Vendor=cvIntel then begin
              for j:=1 to 16 do
                s:=s+Format('%2.2x ',[Descriptors[j]]);
              Add(Format('Level2 Descriptors = %s',[s]));
            end;
          end;
          s:='';
          with Cache.Level3 do begin
            if Vendor=cvIntel then begin
              for j:=1 to 16 do
                s:=s+Format('%2.2x ',[Descriptors[j]]);
              Add(Format('Level3 Descriptors = %s',[s]));
            end;
          end;
          s:='';
          with Cache.Trace do begin
            if Vendor=cvIntel then begin
              for j:=1 to 16 do
                s:=s+Format('%2.2x ',[Descriptors[j]]);
              Add(Format('Trace Descriptors = %s',[s]));
            end;
          end;
          Add('');
          Add('[Standard Features]');
          for j:=0 to Features.Standard.Count-1 do
            with Features.Standard.Features[j],Definition do
              Add(Format('%s (%s) = %d',[Name,Desc,Integer(Value)]));
          Add('');
          Add('[Extended Features]');
          for j:=0 to Features.Extended.Count-1 do
            with Features.Extended.Features[j],Definition do
              Add(Format('%s (%s) = %d',[Name,Desc,Integer(Value)]));
          Add('');
          Add('[Power Management Features]');
          for j:=0 to Features.PowerManagement.Count-1 do
            with Features.PowerManagement.Features[j],Definition do
              Add(Format('%s (%s) = %d',[Name,Desc,Integer(Value)]));
          Add('');
          Add('[Secure Virtual Machine]');
          for j:=0 to Features.SecureVirtualMachine.Count-1 do
            with Features.SecureVirtualMachine.Features[j],Definition do
              Add(Format('%s (%s) = %d',[Name,Desc,Integer(Value)]));
          Add('');

          fn:=f+'.sif';
          h:=True;
          if cbxSave.Checked then
            SaveToStorage(ExtractFilePath(Application.EXEName)+fn,h);

          fn:=f+'.xml';
          if cbxXML.Checked then begin
            sl:=TStringList.Create;
            try
              CPU_XML_Report(CPU,True,sl);
              sl.SaveToFile(ExtractFilePath(Application.EXEName)+fn);
              SaveXSLTemplate(ExtractFilePath(Application.EXEName)+XSLName);
            finally
              sl.Free;
            end;
          end;
          fn:=f+'.sif';

        finally
          Free;
        end;
      except on e: exception do
        Add('ERROR: '+e.Message);
      end;

      Add('[CPUID]');
      try
        CPUID:=ExecuteCPUID(CPU.CPUIndex,CPUID_STD_MaximumLevel);
        m:=CPUID.EAX;
        for j:=0 to m do begin
          CPUID:=ExecuteCPUID(-1,j);
          with CPUID do
            Add(Format('%8.8x = EAX:%8.8x  EBX:%8.8x  ECX:%8.8x  EDX:%8.8x',[CPUIDCommand,EAX,EBX,ECX,EDX]));
          if j=11 then begin
            CPUID:=ExecuteCPUID(-1,j,1,1);
            with CPUID do
              Add(Format('%8.8x = EAX:%8.8x  EBX:%8.8x  ECX:%8.8x  EDX:%8.8x',[CPUIDCommand,EAX,EBX,ECX,EDX]));
            CPUID:=ExecuteCPUID(-1,j,1,2);
            with CPUID do
              Add(Format('%8.8x = EAX:%8.8x  EBX:%8.8x  ECX:%8.8x  EDX:%8.8x',[CPUIDCommand,EAX,EBX,ECX,EDX]));
          end;
        end;

        Add('');

        CPUID:=ExecuteCPUID(CPU.CPUIndex,CPUID_EXT_MaximumLevel);
        m:=CPUID.EAX-CPUID_EXT_MaximumLevel;
        for j:=0 to m do begin
          CPUID:=ExecuteCPUID(CPU.CPUIndex,j+CPUID_EXT_MaximumLevel);
          with CPUID do
            Add(Format('%8.8x = EAX:%8.8x  EBX:%8.8x  ECX:%8.8x  EDX:%8.8x',[CPUIDCommand,EAX,EBX,ECX,EDX]));
        end;
      except on e: exception do
        Add('ERROR: '+e.Message);
      end;

      Add('');
      Add('[Logical Processor Information]');
      try
        n:=0;
        i:=0;
        if Assigned(GetLogicalProcessorInformation) and not GetLogicalProcessorInformation(nil,n) then begin
          buf1:=AllocMem(n);
          try
            if GetLogicalProcessorInformation(buf1,n) then begin
              slpi:=buf1;
              while (NativeUInt(slpi)-NativeUInt(buf1))<n do begin
                case slpi.Relationship of
                  RelationProcessorCore: Add(Format('[%d] RelationProcessorCore: %d bits set',[i,CountSetBits(slpi.ProcessorMask)]));
                  RelationProcessorPackage: Add(Format('[%d] RelationProcessorPackage: %d bits set',[i,CountSetBits(slpi.ProcessorMask)]));
                  RelationNumaNode: Add(Format('[%d] RelationNumaNode: %d',[i,slpi.NodeNumber]));
                  RelationCache: with slpi.Cache do Add(Format('[%d] RelationCache: Type=%d, Level=%d, Associativity=%d, LineSize=%d, Size=%d',[i,Integer(_Type),Level,Associativity,LineSize,Size]));
                  RelationGroup: Add(Format('[%d] RelationGroup',[i]));
                  else Add(Format('[%d] (out of range): %d',[i,Integer(slpi.Relationship)]));
                end;
                slpi:=PSystemLogicalProcessorInformation(NativeUInt(slpi)+SizeOf(TSystemLogicalProcessorInformation));
                inc(i);
              end;
            end else
              Add('Not available');
          finally
            Freemem(buf1);
          end;
        end else
          Add('Not available');
      except on e: exception do
        Add('ERROR: '+e.Message);
      end;

      Add('');
      Add('[Logical Processor Information Ex]');
      try
        n:=0;
        i:=0;
        if Assigned(GetLogicalProcessorInformationEx) and not GetLogicalProcessorInformationEx(RelationAll,nil,n) then begin
          buf2:=AllocMem(n);
          try
            if GetLogicalProcessorInformationEx(RelationAll,buf2,n) then begin
              slpiex:=buf2;
              while (NativeUInt(slpiex)-NativeUInt(buf2))<n do begin
                case slpiex.Relationship of
                  RelationProcessorCore: begin
                    m:=0;
                    for j:=0 to slpiex.Processor.GroupCount-1 do
                      inc(m,CountSetBits(slpiex.Processor.GroupMask[j].Mask));
                    Add(Format('[%d] RelationProcessorCore: %d bits set in %d groups',[i,m,slpiex.Processor.GroupCount]));
                  end;
                  RelationProcessorPackage: begin
                    m:=0;
                    for j:=0 to slpiex.Processor.GroupCount-1 do
                      inc(m,CountSetBits(slpiex.Processor.GroupMask[j].Mask));
                    Add(Format('[%d] RelationProcessorPackage: %d bits set in %d groups',[i,m,slpiex.Processor.GroupCount]));
                  end;
                  RelationNumaNode: Add(Format('[%d] RelationNumaNode: %d - %d bits set in group %d',[i,slpiex.NumaNode.NodeNumber,CountSetBits(slpiex.NumaNode.GroupMask.Mask),slpiex.NumaNode.GroupMask.Group]));
                  RelationCache: with slpiex.Cache do Add(Format('[%d] RelationCache: Type=%d, Level=%d, Associativity=%d, LineSize=%d, Size=%d',[i,Integer(_Type),Level,Associativity,LineSize,CacheSize]));
                  RelationGroup: begin
                    m:=0;
                    for j:=0 to slpiex.Group.ActiveGroupCount-1 do
                      inc(m,CountSetBits(slpiex.Group.GroupInfo[j].ActiveProcessorMask));
                    Add(Format('[%d] RelationGroup: %d bits set in %d active groups',[i,m,slpiex.Group.ActiveGroupCount]));
                  end
                  else Add(Format('[%d] (out of range): %d',[i,Integer(slpiex.Relationship)]));
                end;
                slpiex:=PSystemLogicalProcessorInformationEx(NativeUInt(slpiex)+slpiex.Size);
                inc(i);
              end;
            end else
              Add('Not available');
          finally
            FreeMem(buf2);
          end;
        end else
          Add('Not available');
      except on e: exception do
        Add('ERROR: '+e.Message);
      end;

      if cbxSMBIOS.Checked then begin
        Add('');
        Add('[SMBIOS]');
        with TMiTeC_SMBIOS.Create(Self) do
          try
            Refreshdata;
            for i:=0 to ProcessorCount-1 do begin
              Add(Format('Vendor[%d] = %s',[i,Processor[i].Manufacturer]));
              Add(Format('NameString[%d] = %s',[i,Processor[i].Version]));
              Add(Format('Socket[%d] = %s',[i,Processor[i].Socket+'('+Upgrades[Processor[i].Upgrade]+')']));
              Add(Format('Voltage[%d] = %1.1f V',[i,Processor[i].Voltage]));
              Add(Format('Frequency[%d] = %d MHz',[i,Processor[i].Frequency]));
              Add(Format('ExternalClock[%d] = %d MHz',[i,Processor[i].ExternalClock]));
              Add(Format('CoreCount[%d] = %d',[i,Processor[i].CoreCount]));
              Add(Format('CoreEnabled[%d] = %d',[i,Processor[i].CoreEnabled]));
              Add(Format('ThreadCount[%d] = %d',[i,Processor[i].ThreadCount]));
            end;
            for i:=0 to CacheCount-1 do
              Add(Format('%s = %d of %d KB - %d ns (%s,%s,%s)',[Cache[i].Designation,
                                                                     Cache[i].InstalledSize,
                                                                     Cache[i].MaxSize,
                                                                     Cache[i].Speed,
                                                                     CacheTypes[Cache[i].Typ],
                                                                     CacheAssociativities[Cache[i].Associativity],
                                                                     SRAMTypes[Cache[i].SRAMType]]));


          h:=True;
          if cbxSave.Checked then
            SaveToStorage(ExtractFilePath(Application.EXEName)+fn,h);
        finally
          Free;
        end;
      end;

      if cbxWMI.Checked then begin
        Add('');
        Add('[WMI]');
        try
          if not WMIConnect('','','',Rootnamespace,wmiServices) then
            Exit;
          WMICommand(wmiServices,'Win32_Processor',wmi);

          for i:=0 to High(wmi) do
            for j:=0 to High(wmi[i]) do
              Add(Format('%s[%d] = %s',[wmi[i][j].Name,i,wmi[i][j].Value]));

          if cbxSave.Checked then
            WmiSaveToStorage('Win32_Processor',ExtractFilePath(Application.EXEName)+fn,wmi);
        finally
          WMIDisconnect(wmiServices);
          Finalize(wmi);
        end;
      end;

      Add('');

      Add('Test passed.');
      fn:=Format('%s.txt',[f]);

    try
      SaveToFile(ExtractFilePath(Application.EXEName)+fn);
    except
    end;
  end;
  Box.SelStart:=0;
  Box.SelLength:=0;
end;

procedure Twnd_cputest_Main.Button2Click(Sender: TObject);
begin
  Screen.Cursor:=crHourglass;
  Box.Lines.Clear;
  DetectCPU;
  Screen.Cursor:=crDefault;
end;


end.
