unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, StdCtrls, MiTeC_CommonDefs, MiTeC_NativeDefs, MSI_SysProcMon;

type
  TwndMain = class(TForm)
    List: TListView;
    sb: TStatusBar;
    InfoPanel: TPanel;
    Panel2: TPanel;
    lTitle: TLabel;
    CPUPanel: TPanel;
    CPUGauge: TProgressBar;
    CPUPanelTitle: TPanel;
    lPri: TLabel;
    lCT: TLabel;
    lMem: TLabel;
    lCPU: TLabel;
    lIORead: TLabel;
    lIOWrite: TLabel;
    ListPanel: TPanel;
    eSearch: TEdit;
    ThreadList: TListView;
    pc: TPageControl;
    tsThreads: TTabSheet;
    tsModules: TTabSheet;
    tsHandles: TTabSheet;
    ModList: TListView;
    HandleList: TListView;
    lPub: TLabel;
    lSigner: TLabel;
    tsEnv: TTabSheet;
    mEnv: TMemo;
    tsGDIH: TTabSheet;
    GDIHandleList: TListView;
    lUser: TLabel;
    lE: TLabel;
    lDPIA: TLabel;
    lPSP: TLabel;
    tsMP: TTabSheet;
    lDEP: TLabel;
    lASLR: TLabel;
    lCFG: TLabel;
    lSP: TLabel;
    lDesktop: TLabel;
    lCmd: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure eSearchKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ListSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure SysProcMonThreadInterval(Sender: TSysProcMonThread);
    procedure lPubClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FSPM: TSysProcMonThread;
    FRecord: TProcessRecord;
    procedure RefreshProcList;
    procedure FindProcess(const AText: string);
    procedure RefreshThreadList;
    procedure RefreshModList;
    procedure RefreshHandleList;
    procedure RefreshEnvList;
    procedure RefreshGDIHandleList;
  public
  end;

var
  wndMain: TwndMain;

implementation

uses MiTeC_StrUtils, MiTeC_Routines, MiTeC_NativeAPI, MiTeC_Datetime,
  Math, MiTeC_Cert, MiTeC_WinTrust, MiTeC_Windows;

{$R *.dfm}

{ TwndMain }

procedure TwndMain.eSearchKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift=[]) and (Key=vk_return) then
    FindProcess(eSearch.Text);
end;

procedure TwndMain.FindProcess;
var
  n: TListItem;
begin
  if Trim(AText)='' then
    Exit;
  for n in List.Items do
    if PosText(AText,n.SubItems[2])>0 then begin
      List.Selected:=n;
      List.Selected.MakeVisible(False);
      Break;
    end;
end;

procedure TwndMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FSPM.OnInterval:=nil;
  FSPM.Terminate;
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  EnablePrivilege('SeDebugPrivilege');
  ResetMemory(FRecord,sizeof(FRecord));
  FSPM:=TSysProcMonThread.Create;
  //FSPM.EnumerateHandles:=True;
  FSPM.EnumerateModules:=True;
  FSPM.EnumerateThreads:=True;
  FSPM.EvaluateSecurityInfo:=True;
  FSPM.RetrieveProcessIcon:=False;
  FSPM.OnInterval:=SysProcMonThreadInterval;
  FSPM.Suspended:=False;
  pc.ActivePageIndex:=0;
end;

procedure TwndMain.ListSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  if not Selected or not Assigned(Item) then
    Exit;
  lSigner.Tag:=0;
  FSPM.SetWatchedProcess(FRecord.PID,False);
  FSPM.SetWatchedProcess(StrToInt(Item.Caption),True);
  FSPM.GetRecordByPID(StrToInt(Item.Caption),FRecord);
  InfoPanel.Show;
end;

procedure TwndMain.lPubClick(Sender: TObject);
begin
  if TLabel(Sender).Font.Style<>[] then
    VerifyFile(FRecord.ImageName,Handle);
end;

procedure TwndMain.SYsProcMonThreadInterval(Sender: TSysProcMonThread);
var
  p: TProcessRecord;
  s,pn,pl,ml: string;
  dt: TDatetime;
  vi: TVersionInfo;
begin
  RefreshProcList;
  Sender.GetRecordByPID(Sender.MaxCPUUsageProcessID,p);
  sb.Panels[1].Text:=Format('Max CPU usage: %s (%d)',[p.Name,p.PID]);

  if FSPM.IsProcessWatched(FRecord.PID) then begin
    FSPM.GetRecordByPID(FRecord.PID,FRecord);
    if lSigner.Tag=0 then begin
      if VerifyFile(FRecord.ImageName,s,dt,pn,pl,ml)=0 then begin
        lSigner.Caption:='(Verified) '+s;
        lSigner.Font.Color:=clBlue;
        lSigner.Font.Style:=[fsUnderline];
        lSigner.Cursor:=crHandPoint;
      end else begin
        GetFileVerInfo(FRecord.ImageName,vi);
        lSigner.Caption:='(UNVERIFIED) '+vi.CompanyName;
        lSigner.Font.Color:=clWindowText;
        lSigner.Font.Style:=[];
        lSigner.Cursor:=crDefault;
      end;
      lSigner.Tag:=1;
    end;
    CPUGauge.Position:=FRecord.Performance.CPUUsage.RoundValue;
    lCPU.Caption:=Format('CPU Usage: %1.2f %%',[FRecord.Performance.CPUUsage.Value]);
    lTitle.Caption:=Format('[%d] %s (%d-bit)',[FRecord.PID,FRecord.ImageName,FRecord.Platform]);
    lCT.Caption:=Format('Created: %s',[DateTimeToStr(FRecord.CreationTime)]);
    lPri.Caption:=Format('Priority: %d',[FRecord.Priority]);
    lMem.Caption:=Format('Memory: %s',[NormalizeFloatDataValue(FRecord.VMCounters.WorkingSetSize/1)]);
    lIORead.Caption:=Format('IO Read Rate: %s/s (Total: %s)',[NormalizeFloatDataValue(FRecord.Performance.IOReadUsage.Delta),NormalizeFloatDataValue(FRecord.Performance.IOReadUsage.Value)]);
    lIOWrite.Caption:=Format('IO Write Rate: %s/s (Total: %s)',[NormalizeFloatDataValue(FRecord.Performance.IOWriteUsage.Delta),NormalizeFloatDataValue(FRecord.Performance.IOWriteUsage.Value)]);
    lE.Caption:=Format('Elevation: %s',[FRecord.ElevationStr]);
    lDPIA.Caption:=Format('DPI Awareness: %s',[GetDpiAwarenessStr(FRecord.DpiAwareness)]);
    lPSP.Caption:=Format('Protection: %s',[FRecord.ProtectionStr]);
    lDesktop.Caption:=Format('Desktop: %s',[FRecord.Desktop]);
    lCmd.Caption:=Format('Command line: %s',[FRecord.CommandLine]);

    lDEP.Caption:=Format('Data Execution Protection: %s',[FRecord.DEPStr]);
    lASLR.Caption:=Format('Address Space Location Randomization: %s',[FRecord.ASLRStr]);
    lCFG.Caption:=Format('Control Flow Guard: %s',[FRecord.CFGuardStr]);
    lSP.Caption:=Format('Stack Protection: %s',[FRecord.StackProtectionStr]);

    s:=FRecord.UserName;
    if FRecord.DomainName<>'' then
      s:=FRecord.DomainName+'\'+s;
    lUser.Caption:=Format('Username: %s',[s]);

    tsHandles.Caption:=Format('Handles (%d/%d)',[FRecord.HandleList.Count, FRecord.HandleCount]);
    tsModules.Caption:=Format('Modules (%d)',[FRecord.ModuleList.Count]);
    RefreshThreadList;
    RefreshEnvList;
    if tsModules.TabVisible and ((pc.ActivePage=tsModules) or (ModList.Items.Count=0)) then
      RefreshModList;
    if (pc.ActivePage=tsHandles) or (HandleList.Items.Count=0) then
      RefreshHandleList;
    RefreshGDIHandleList;
  end;
end;

procedure TwndMain.RefreshEnvList;
begin
  mEnv.Text:=FRecord.Environment;
  tsEnv.Caption:=Format('Environment (%d)',[mEnv.Lines.Count]);
end;

procedure TwndMain.RefreshGDIHandleList;
var
  tid: int64;
  vp,i: Integer;
  SInfo: TScrollInfo;
  r: TRect;
  h: TGDIHandleRecord;
begin
  tid:=-1;
  if Assigned(GDIHandleList.Selected) then
    tid:=StrToInt(GDIHandleList.Selected.Caption);
  SInfo.cbSize:=SizeOf(SInfo);
  SInfo.fMask:=SIF_ALL;
  GetScrollInfo(GDIHandleList.Handle,SB_VERT,SInfo);
  vp:=SInfo.nPos;
  GDIHandleList.Items.BeginUpdate;
  try
    GDIHandleList.Items.Clear;
    for i:=0 to FRecord.GDIHandleList.Count-1 do begin
      h:=PGDIHandleRecord(FRecord.GDIHandleList[i])^;
      with GDIHandleList.Items.Add do begin
        Caption:=h.TypeName;
        SubItems.Add(Format('0x%x',[h.Handle]));
        Subitems.Add(Format('0x%x',[NativeUInt(h.Entry.KernelAddress)]));
        SubItems.Add(h.Information);
      end;
    end;
  finally
    GDIHandleList.Items.EndUpdate;
  end;
  if GDIHandleList.Items.Count>0 then begin
    GDIHandleList.Items.BeginUpdate;
    try
      r:=GDIHandleList.Items[0].DisplayRect(drBounds);
      GDIHandleList.Scroll(0,vp*(r.Bottom-r.Top));
      if tid<>-1 then
        GDIHandleList.Selected:=GDIHandleList.FindCaption(0,IntToStr(tid),False,True,True);
    finally
      GDIHandleList.Items.EndUpdate;
    end;
  end;
  tsGDIH.Caption:=Format('GDI Handles (%d)',[GDIHandleList.Items.Count]);
end;

procedure TwndMain.RefreshHandleList;
var
  hid: THandle;
  vp,i: Integer;
  SInfo: TScrollInfo;
  r: TRect;
  h: THandleRecord;
begin
  hid:=0;
  if Assigned(HandleList.Selected) then
    hid:=StrToInt('$'+Copy(HandleList.Selected.Caption,3,Length(HandleList.Selected.Caption)));
  SInfo.cbSize:=SizeOf(SInfo);
  SInfo.fMask:=SIF_ALL;
  GetScrollInfo(HandleList.Handle,SB_VERT,SInfo);
  vp:=SInfo.nPos;
  HandleList.Items.BeginUpdate;
  try
    HandleList.Items.Clear;
    for i:=0 to FRecord.HandleList.Count-1 do begin
      h:=PHandleRecord(FRecord.HandleList[i])^;
      with HandleList.Items.Add do begin
        Caption:=Format('0x%x',[h.Handle]);
        SubItems.Add(Format('%s',[h.TypeName]));
        Subitems.Add(h.Name);
      end;
    end;
  finally
    HandleList.Items.EndUpdate;
  end;
  if HandleList.Items.Count>0 then begin
    HandleList.Items.BeginUpdate;
    try
      r:=HandleList.Items[0].DisplayRect(drBounds);
      HandleList.Scroll(0,vp*(r.Bottom-r.Top));
      if hid<>0 then
        HandleList.Selected:=HandleList.FindCaption(0,Format('0x%x',[hid]),False,True,True);
    finally
      HandleList.Items.EndUpdate;
    end;
  end;
end;

procedure TwndMain.RefreshModList;
var
  mid: string;
  vp,i: Integer;
  SInfo: TScrollInfo;
  r: TRect;
  m: TModuleRecord;
begin
  mid:='';
  if Assigned(ModList.Selected) then
    mid:=ModList.Selected.Caption;
  SInfo.cbSize:=SizeOf(SInfo);
  SInfo.fMask:=SIF_ALL;
  GetScrollInfo(ModList.Handle,SB_VERT,SInfo);
  vp:=SInfo.nPos;
  ModList.Items.BeginUpdate;
  try
    ModList.Items.Clear;
    for i:=0 to FRecord.ModuleList.Count-1 do begin
      m:=PModuleRecord(FRecord.ModuleList[i])^;
      with ModList.Items.Add do begin
        Caption:=m.Name;
        SubItems.Add(Format('0x%8.8x',[m.BaseAddress]));
        Subitems.Add(Format('%d KB',[m.ImageSize shr 10]));
        SubItems.Add(m.VersionInfo.Description);
      end;
    end;
  finally
    ModList.Items.EndUpdate;
  end;
  if ModList.Items.Count>0 then begin
    ModList.Items.BeginUpdate;
    try
      r:=ModList.Items[0].DisplayRect(drBounds);
      ModList.Scroll(0,vp*(r.Bottom-r.Top));
      if mid<>'' then
        ModList.Selected:=ModList.FindCaption(0,mid,False,True,True);
    finally
      ModList.Items.EndUpdate;
    end;
  end;
end;

procedure TwndMain.RefreshProcList;
var
  i: Integer;
  vp: Integer;
  SInfo: TScrollInfo;
  r: TRect;
  p: TProcessRecord;
begin
  SInfo.cbSize:=SizeOf(SInfo);
  SInfo.fMask:=SIF_ALL;
  GetScrollInfo(List.Handle,SB_VERT,SInfo);
  vp:=SInfo.nPos;
  List.Items.BeginUpdate;
  try
    List.Items.Clear;
    for i:=0 to FSPM.RecordCount-1 do begin
      FSPM.GetRecord(i,p);
      with List.Items.Add do begin
        Caption:=IntToStr(p.PID);
        SubItems.Add(IntToStr(p.SessionID));
        SubItems.Add(IntToStr(p.Platform));
        if CompareValue(p.Performance.CPUUsage.Value,0,0.01)>0 then
          SubItems.Add(Format('%1.2f',[p.Performance.CPUUsage.Value]))
        else
          SubItems.Add('');
        SubItems.Add(NormalizeFloatDataValue(p.VMCounters.WorkingSetSize/1));
        SubItems.Add(p.Name);
        if p.DomainName<>'' then
          SubItems.Add(p.DomainName+'\'+p.UserName)
        else
          SubItems.Add(p.UserName);
      end;
    end;
  finally
    List.Items.EndUpdate;
  end;
  if List.Items.Count>0 then begin
    List.Items.BeginUpdate;
    try
      r:=List.Items[0].DisplayRect(drBounds);
      List.Scroll(0,vp*(r.Bottom-r.Top));
      List.OnSelectItem:=nil;
      try
        List.Selected:=List.FindCaption(0,IntToStr(FRecord.PID),False,True,True);
      finally
        List.OnSelectItem:=ListSelectItem;
      end;
    finally
      List.Items.EndUpdate;
    end;
  end;
  sb.Panels[0].Text:=Format('%d item(s)',[List.Items.Count]);
end;

procedure TwndMain.RefreshThreadList;
var
  tid: int64;
  vp,i: Integer;
  SInfo: TScrollInfo;
  r: TRect;
  t: TThreadRecord;
begin
  tid:=-1;
  if Assigned(ThreadList.Selected) then
    tid:=StrToInt(ThreadList.Selected.Caption);
  SInfo.cbSize:=SizeOf(SInfo);
  SInfo.fMask:=SIF_ALL;
  GetScrollInfo(ThreadList.Handle,SB_VERT,SInfo);
  vp:=SInfo.nPos;
  ThreadList.Items.BeginUpdate;
  try
    ThreadList.Items.Clear;
    for i:=0 to FRecord.ThreadList.Count-1 do begin
      t:=PThreadRecord(FRecord.ThreadList[i])^;
      with ThreadList.Items.Add do begin
        Caption:=IntToStr(t.ID);
        SubItems.Add(Format('%s:%s',[cThreadState[TThreadState(t.State)],cKWaitReason[TKWaitReason(t.WaitReason)]]));
        if CompareValue(t.CPUUsage,0,0.01)>0 then
          SubItems.Add(Format('%1.2f',[t.CPUUsage]))
        else
          Subitems.Add('');
        SubItems.Add(Format('%d / %d',[t.Priority,t.BasePriority]));
        SubItems.Add(t.StartAddressString);
      end;
    end;
  finally
    ThreadList.Items.EndUpdate;
  end;
  if ThreadList.Items.Count>0 then begin
    ThreadList.Items.BeginUpdate;
    try
      r:=ThreadList.Items[0].DisplayRect(drBounds);
      ThreadList.Scroll(0,vp*(r.Bottom-r.Top));
      if tid<>-1 then
        ThreadList.Selected:=ThreadList.FindCaption(0,IntToStr(tid),False,True,True);
    finally
      ThreadList.Items.EndUpdate;
    end;
  end;
  tsThreads.Caption:=Format('Threads (%d)',[ThreadList.Items.Count]);
end;

end.
