unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  ExtCtrls;

type
  TwndMain = class(TForm)
    Label1: TLabel;
    eFile: TEdit;
    bOpen: TButton;
    od: TOpenDialog;
    Memo: TMemo;
    bView: TButton;
    bProps: TButton;
    Image: TImage;
    procedure bOpenClick(Sender: TObject);
    procedure bViewClick(Sender: TObject);
    procedure bPropsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FPwd: string;
  public
    procedure OpenFile(const AFilename: string);
  end;

var
  wndMain: TwndMain;

implementation

uses MiTeC_CommonDefs, MiTeC_Cert, MiTeC_Routines, MiTeC_Dialogs, MiTeC_WinCrypt;

{$R *.dfm}

procedure TwndMain.bOpenClick(Sender: TObject);
begin
  if not od.Execute then
    Exit;
  OpenFile(od.Filename);
end;

procedure TwndMain.bPropsClick(Sender: TObject);
begin
 ShellPropDlg(Handle,eFile.Text);
end;

procedure TwndMain.bViewClick(Sender: TObject);
var
  e: string;
begin
  e:=LowerCase(ExtractFileExt(eFile.Text));
  if IsFilePE(eFile.Text) then
    VerifyFile(eFile.text,Handle)
  else if Pos(e,'.pem.p7c.p7b.cer.crt.der')>0 then
    ViewCertificate(Handle,eFile.text)
  else if Pos(e,'.pfx.p12')>0 then
    ViewPFXCertificate(Handle,eFile.text,FPwd);
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  Image.Picture.Icon.Handle:=Application.Icon.Handle;
end;

procedure TwndMain.FormShow(Sender: TObject);
begin
  if ParamCount>0 then
    OpenFile(ParamStr(1));
end;

procedure TwndMain.OpenFile(const AFilename: string);
var
  e: string;
  csi: TCertStoreInfo;
  ci: TCertContextInfo;
  cs: HCERTSTORE;
  vi: TVersionInfo;
  r,i: integer;
  si: PCmsgSignerInfo;
begin
  FPwd:='';
  bProps.Enabled:=True;
  Memo.Clear;
  Memo.Lines.BeginUpdate;
  try
    bView.Enabled:=False;
    eFile.Text:=AFilename;
    cs:=nil;
    e:=LowerCase(ExtractFileExt(AFilename));
    if IsFilePE(eFile.Text) then begin
      GetFileVerInfo(AFilename,vi);
      Memo.Lines.Add('Company: '+vi.CompanyName);
      Memo.Lines.Add('Description: '+vi.Description);
      Memo.Lines.Add('Product: '+vi.ProductName);
      Memo.Lines.Add('Prod version: '+vi.ProductVersion);
      Memo.Lines.Add('File version: '+vi.FileVersion);
      Memo.Lines.Add('Copyright: '+vi.Copyright);
      Memo.Lines.Add('');
      if GetCertStoreFromSignedFile(AFilename,cs,si) and Assigned(cs) then begin
        r:=VerifyFile(AFilename,ci.Subject,ci.ValidFrom,ci.ProgramName,ci.PublisherLink,ci.MoreInfoLink);
        if r<>0 then begin
          Memo.Lines.Add('ERROR: '+GetCertErrorText(r));
          Memo.Lines.Add('');
        end else begin
          Memo.Lines.Add('Publisher: '+ci.Subject);
          Memo.Lines.Add('Program name: '+ci.ProgramName);
          Memo.Lines.Add('Publisher link: '+ci.PublisherLink);
          Memo.Lines.Add('More info link: '+ci.MoreInfoLink);
          if ci.ValidFrom>0 then
            Memo.Lines.Add('Signing date: '+DateTimeToStr(ci.ValidFrom));
          Memo.Lines.Add('');
        end;
      end else begin
        Memo.Lines.Add('File is not signed');
        Memo.Lines.Add('');
      end;
    end else if Pos(e,'.pem.p7c.p7b.cer.crt.der')>0 then begin
      if not GetCertStore(AFilename,cs) or not Assigned(cs) then begin
        Memo.Lines.Add('File contains no certificate');
        Exit;
      end;
    end else if Pos(e,'.pfx.p12')>0 then begin
      if InputQuery('Password','',FPwd) then
        if not GetPFXCertStore(AFilename,FPwd,cs) or not Assigned(cs) then begin
          Memo.Lines.Add('File contains no certificate or password is not valid');
          Exit;
        end;
    end else
      Memo.Lines.Add('File type is not supported');

    if Assigned(cs) then begin
      bView.Enabled:=True;
      RetrieveCertStoreInfo(cs,si,csi);
      if Assigned(si) then
        Freemem(si);
      CertCloseStore(cs,0);

      for i:=0 to High(csi) do begin
        Memo.Lines.Add(Format('Certificate %s version %d',[csi[i].Encoding,csi[i].Version]));
        Memo.Lines.Add('Issuer: '+csi[i].Issuer);
        Memo.Lines.Add('Subject: '+csi[i].Subject);
        Memo.Lines.Add('Serial: '+csi[i].Serial);
        Memo.Lines.Add('SignatureAlgorithm: '+csi[i].SignatureAlgorithm);
        Memo.Lines.Add('PublicKeyAlgorithm: '+csi[i].PublicKeyAlgorithm);
        Memo.Lines.Add('PublicKeyBits: '+IntToStr(csi[i].PublicKeyBits));
        Memo.Lines.Add('ValidFrom: '+DateTimeToStr(csi[i].ValidFrom));
        Memo.Lines.Add('ValidTo: '+DateTimeToStr(csi[i].ValidTo));
        Memo.Lines.Add(Format('Key usage: %s',[csi[i].KeyUsage]));
        Memo.Lines.Add(Format('Enhanced key usage: %s',[csi[i].EnhancedKeyUsage]));
        Memo.Lines.Add(Format('Thumbprint: %s',[csi[i].Thumbprint]));
        Memo.Lines.Add(StringOfChar('-',80));
      end;
    end;
    Memo.SelStart:=0;
    Memo.SelLength:=0;
  finally
    Memo.Lines.EndUpdate;
  end;
end;

end.
