unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, MSI_Monitor, ExtCtrls, StdCtrls, MiTeC_MultiMon;

type
  TwndMain = class(TForm)
    scb: TScrollBox;
    cbMon: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MonClick(Sender: TObject);
    procedure cbMonSelect(Sender: TObject);
  private
    FMon: TMiTeC_Monitor;
    procedure SetMonitors;
  protected
    procedure WMDISPLAYCHANGE(var Message: TWMDisplayChange); message WM_DISPLAYCHANGE;
    procedure WMDPICHANGED(var Message: TWMDpi); message WM_DPICHANGED;
  public
    procedure RefreshData;
  end;

var
  wndMain: TwndMain;

implementation

uses MiTeC_Helpers;

{$R *.dfm}

{ TwndMain }

procedure TwndMain.SetMonitors;
var
  i,x,y: integer;
  z: double;
  m: TMonitorRecord;
  p: TPanel;
  l: TLabel;
  s: string;
begin
  cbMon.Items.Clear;
  while scb.ControlCount>0 do
    scb.Controls[0].Free;

  z:=7*96/MiTeC_MultiMon.GetPixelsPerInch(Screen.MonitorFromWindow(Self.Handle,mdNearest).Handle);
  x:=0;
  y:=0;
  for i:=0 to FMon.Count-1 do begin
    if FMon[i].Bounds.Left<x then
      x:=FMon[i].Bounds.Left;
    if FMon[i].Bounds.Top<y then
      y:=FMon[i].Bounds.Top;
  end;

  x:=abs(x);
  y:=abs(y);
  for i:=0 to FMon.Count-1 do begin
    m:=FMon.Monitors[i];

    cbMon.Items.Add(Format('%s - %s',[m.DeviceID,m.Model]));

    p:=TPanel.Create(Self);
    p.BevelOuter:=bvNone;
    p.BevelKind:=bkFlat;
    p.Font.Size:=20;
    p.Font.Color:=clGray;
    p.Font.Style:=[fsBold];
    p.ParentBackground:=False;
    if m.Primary then
      p.Color:=clCream;

    p.Parent:=scb;
    p.Width:=Round(RectWidth(m.Bounds)/z)-3;
    p.Height:=Round(RectHeight(m.Bounds)/z)-3;
    p.Top:=Round((m.Bounds.Top+y)/z)+3;
    p.Left:=Round((m.Bounds.Left+x)/z)+3;
    p.Caption:=IntToStr(m.MonitorNum+1);
    p.Tag:=i;
    p.OnClick:=MonClick;
    p.Cursor:=crHandpoint;

    l:=TLabel.Create(Self);
    l.AlignWithMargins:=True;
    l.Alignment:=taCenter;
    l.WordWrap:=True;
    l.ParentFont:=False;
    l.Parent:=p;
    l.Align:=alTop;
    {$IFDEF DEBUG}
    l.Caption:=Format('%s - %d'#13#10'%s'#13#10'(%d x %d) cm',[m.DeviceID,m.MonitorNum,m.DeviceDescription,m.EDIDRecord.Width,m.EDIDRecord.Height]);
    {$ELSE}
    l.Caption:=Format('%s'#13#10'(%d x %d) cm',[m.DeviceDescription,m.EDIDRecord.Width,m.EDIDRecord.Height]);
    {$ENDIF}
    l.Tag:=i;
    l.OnClick:=MonClick;
    l.Cursor:=crHandpoint;

    l:=TLabel.Create(Self);
    l.AlignWithMargins:=True;
    l.Alignment:=taCenter;
    l.WordWrap:=True;
    l.ParentFont:=False;
    l.Parent:=p;
    l.Align:=alBottom;
    s:=m.EDIDRecord.Name;
    if s='' then
      s:=m.Model;
    l.Caption:=Format('%s'#13#10'(%d x %d) px - %d dpi',[s,RectWidth(m.Bounds),RectHeight(m.Bounds),m.DPI]);
    l.Tag:=i;
    l.OnClick:=MonClick;
    l.Cursor:=crHandpoint;
  end;

  Caption:=Format('Monitor View (%d)',[FMon.Count]);
end;

procedure TwndMain.cbMonSelect(Sender: TObject);
var
  i: integer;
begin
  for i:=0 to scb.ControlCount-1 do
    if (scb.Controls[i].Tag=cbMon.ItemIndex) then begin
      scb.Controls[i].BringToFront;
      Break;
    end;
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  FMon:=TMiTeC_Monitor.Create(Self);
end;

procedure TwndMain.FormShow(Sender: TObject);
begin
  RefreshData;
end;

procedure TwndMain.MonClick(Sender: TObject);
begin
  cbMon.ItemIndex:=TComponent(Sender).Tag;
end;

procedure TwndMain.RefreshData;
begin
  Screen.Cursor:=crHourglass;
  try
    FMon.RefreshData;
    SetMonitors;
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure TwndMain.WMDISPLAYCHANGE(var Message: TWMDisplayChange);
begin
  RefreshData;
end;

procedure TwndMain.WMDPICHANGED(var Message: TWMDpi);
begin
  RefreshData;
end;

end.
