unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, MiTeC_Firmware;

type
  TwndMain = class(TForm)
    LabelAddress: TLabel;
    LabelLength: TLabel;
    bDump: TButton;
    Memo: TMemo;
    cbTS: TComboBox;
    cbTID: TComboBox;
    bWMI: TButton;
    Label1: TLabel;
    bReg: TButton;
    bROM: TButton;
    bAPI: TButton;
    bLoad: TButton;
    od: TOpenDialog;
    bSave: TButton;
    sd: TSaveDialog;
    procedure bDumpClick(Sender: TObject);
    procedure cbTSChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bSMBIOSClick(Sender: TObject);
    procedure bLoadClick(Sender: TObject);
    procedure bSaveClick(Sender: TObject);
  private
    FSMBIOS: TSMBIOS;
  public
    procedure DisplayData;
  end;

var
  wndMain: TwndMain;

implementation

uses TypInfo, MiTeC_Routines, MiTeC_LSAAPI;

{$R *.dfm}

function GetMachine :string;
var
  n :Cardinal;
  buf :PChar;
const
  rkMachine = {HKEY_LOCAL_MACHINE}'\SYSTEM\CurrentControlSet\Control\ComputerName\ComputerName';
    rvMachine = 'ComputerName';
begin
  n:=255;
  buf:=stralloc(n);
  GetComputerName(buf,n);
  result:=buf;
  strdispose(buf);
end;

function StrToHex(const AValue: string; AReverse: boolean = False): Cardinal;
var
  c: Char;
  s: string;
begin
  s:='';
  for c in AValue do
    if AReverse then
      s:=IntToHex(Ord(c),2)+s
    else
      s:=s+IntToHex(Ord(c),2);
  Result:=StrToIntDef('$'+s,0);
end;

procedure BytesToHexView(const AData: TBytes; AList: TStrings; ARuler: boolean = True);

function GetChar(Value: Char): Char;
begin
  if Value<' ' then
    Result:='.'
  else
    Result:=Value;
end;

var
  i,j,LineCount,p: Integer;
  r,d,a: String;
begin
  LineCount:=Length(AData) div 16;
  if Length(AData) mod 16 > 0 then
    Inc(LineCount);
  for i:=0 to LineCount-1 do begin
    if ARuler then
      r:=IntToHex(i*16,8)+': '
    else
      r:='';
    d:='';
    a:='';
    for j:=0 to 15 do begin
      p:=i*16+j;
      if p<Length(AData) then begin
        d:=d+IntToHex(AData[p],2)+' ';
        a:=a+GetChar(Chr(AData[p]));
      end else begin
        d:=d+StringOfChar(' ',(16-j)*3);
        Break;
      end;
    end;
    AList.Add(r+d+' '+a);
  end;
end;

procedure TwndMain.bDumpClick(Sender: TObject);
var
  b: TBytes;
begin
  Screen.Cursor:=crHourGlass;
  Memo.Lines.BeginUpdate;
  try
    Memo.Lines.Clear;
    FSMBIOS.Clear;
    if not TFirmware.DumpSystemFirmwareTable(StrToHex(cbTS.Text),StrToHex(cbTID.Text,True),b) then
      Exit;
    BytesToHexView(b,Memo.Lines);
    Memo.Lines.Add('');

    if SameText(cbTID.Text,'MSDM') then
      Memo.Lines.Add(Format('Windows Product Key: %s',[TFirmware.WindowsProductKey]))
    else if SameText(cbTID.Text,'UEFI') then begin
      Memo.Lines.Add(Format('GUID: %s',[GUIDToString(TFirmware.UEFIGUID)]));
      Memo.Lines.Add(Format('UEFI Secure Boot: %s',[Copy(GetEnumName(TypeInfo(TUEFISecureBoot),Ord(TFirmware.UEFISecureBoot)),7)]));
    end;
  finally
    Memo.Lines.EndUpdate;
    Screen.Cursor:=crDefault;
  end;
end;

procedure TwndMain.bLoadClick(Sender: TObject);
begin
  if not od.Execute then
    Exit;
  od.InitialDir:=ExtractFilePath(od.Filename);
  sd.InitialDir:=od.InitialDir;
  Screen.Cursor:=crHourGlass;
  Memo.Lines.BeginUpdate;
  try
    Memo.Lines.Clear;
    FSMBIOS.LoadFromFile(od.Filename);
    BytesToHexView(FSMBIOS.Data,Memo.Lines);
    Memo.Lines.Add('');
    DisplayData;
  finally
    Memo.Lines.EndUpdate;
    Screen.Cursor:=crDefault;
  end;
end;

procedure TwndMain.bSaveClick(Sender: TObject);
begin
  if FSMBIOS.Size=0 then
    Exit;
  sd.Filename:=Format('%s_%d%d%d.smbios',[GetMachine,FSMBIOS.MajorVersion,FSMBIOS.MinorVersion,FSMBIOS.DMIRevision]);
  sd.Filename:=StringReplace(sd.Filename,',',' ',[rfReplaceAll]);
  sd.Filename:=StringReplace(sd.Filename,'/','-',[rfReplaceAll]);

  if not sd.Execute then
    Exit;

  sd.InitialDir:=ExtractFilePath(sd.Filename);
  od.InitialDir:=sd.InitialDir;
  FSMBIOS.SaveToFile(sd.Filename);
end;

procedure TwndMain.bSMBIOSClick(Sender: TObject);
begin
  Screen.Cursor:=crHourGlass;
  Memo.Lines.BeginUpdate;
  try
    Memo.Lines.Clear;
    case TComponent(Sender).Tag of
      0: FSMBIOS.LoadFromAPI;
      1: FSMBIOS.LoadFromWMI;
      2: FSMBIOS.LoadFromRegistry;
      3: FSMBIOS.LoadFromROM;
    end;
    BytesToHexView(FSMBIOS.Data,Memo.Lines);
    Memo.Lines.Add('');
    DisplayData;
  finally
    Memo.Lines.EndUpdate;
    Screen.Cursor:=crDefault;
  end;
end;

procedure TwndMain.cbTSChange(Sender: TObject);
begin
  TFirmware.GetSystemFirmwareTables(StrToHex(cbTS.Text),cbTID.Items);
end;

procedure TwndMain.DisplayData;
var
  i: integer;
begin
  for i:=0 to FSMBIOS.TableCount-1 do
    Memo.Lines.Add(Format('%3.3d: %s',[FSMBIOS.Tables[i].Header.Typ,FSMBIOS.Tables[i].Name]));
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  EnablePrivilege(SE_SYSTEM_ENVIRONMENT_NAME);
  TFirmware.GetSystemFirmwareTables(StrToHex(cbTS.Text),cbTID.Items);
  FSMBIOS:=TSMBIOS.Create;
  sd.InitialDir:=ExtractFilePath(Application.EXEName);
  od.InitialDir:=sd.InitialDir;
end;

end.

