unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, Graphics,
  ImgList, Controls, ComCtrls, StdCtrls,
  MiTeC_NCC, MiTeC_NetChangeNotify, ExtCtrls, Dialogs;

type
  TwndMain = class(TForm)
    bOK: TButton;
    List: TListView;
    Timer: TTimer;
    bRefresh: TButton;
    procedure FormCreate(Sender: TObject);
    procedure bOKClick(Sender: TObject);
    procedure NetworkChangeNotifierChange(Sender: TObject; ARes: Cardinal);
    procedure ListAdvancedCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
      var DefaultDraw: Boolean);
    procedure ListAdvancedCustomDrawSubItem(Sender: TCustomListView;
      Item: TListItem; SubItem: Integer; State: TCustomDrawState;
      Stage: TCustomDrawStage; var DefaultDraw: Boolean);
    procedure TimerTimer(Sender: TObject);
    procedure bRefreshClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FNC: TNetworkConfiguration;
    FNetChangeNotifier: TNetworkChangeNotifier;
  public
    procedure DisplayData;
  end;

var
  wndMain: TwndMain;

implementation

uses MiTeC_StrUtils, MiTeC_IPTypes;

{$R *.dfm}

function GetNameCommaText(AList: TStringList): string;
var
  i: integer;
begin
  Result:='';
  for i:=0 to AList.Count-1 do
    Result:=AList.Names[i]+',';
  SetLength(Result,Length(Result)-1);
end;

function GetValueCommaText(AList: TStringList): string;
var
  i: integer;
begin
  Result:='';
  for i:=0 to AList.Count-1 do
    Result:=AList.ValueFromIndex[i]+',';
  SetLength(Result,Length(Result)-1);
end;

procedure TwndMain.bOKClick(Sender: TObject);
begin
  Close;
end;

procedure TwndMain.bRefreshClick(Sender: TObject);
begin
  FNC.RefreshData;
  DisplayData;
end;

procedure TwndMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FNC.Free;
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  FNC:=TNetworkConfiguration.Create;
  FNC.RefreshData;
  DisplayData;
  FNetChangeNotifier:=TNetworkChangeNotifier.Create;
  FNetChangeNotifier.OnChange:=NetworkChangeNotifierChange;
end;


procedure TwndMain.ListAdvancedCustomDrawItem(Sender: TCustomListView;
  Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
  var DefaultDraw: Boolean);
begin
  DefaultDraw:=True;
  with TListView(Sender) do begin
    Canvas.Font.Style:=[];
    Canvas.Font.Color:=Font.Color;
    Canvas.Font.Style:=Canvas.Font.Style+[fsUnderline];
    if not HotTrack or not(htUnderlineCold in HotTrackStyles) then
      Canvas.Font.Style:=Font.Style;
    if (cdsHot in State) and HotTrack then begin
      if htUnderlineHot in HotTrackStyles then
        Canvas.Font.Style:=Canvas.Font.Style+[fsUnderline];
      Canvas.Font.Color:=clHotLight;
    end;
    if Item.ImageIndex=-2 then
      Canvas.Font.Style:=[fsBold];
  end;
end;

procedure TwndMain.ListAdvancedCustomDrawSubItem(Sender: TCustomListView;
  Item: TListItem; SubItem: Integer; State: TCustomDrawState;
  Stage: TCustomDrawStage; var DefaultDraw: Boolean);
begin
  DefaultDraw:=True;
  with TListView(Sender) do begin
    Canvas.Font.Style:=[];
    Canvas.Font.Color:=Font.Color;
    Canvas.Font.Style:=Canvas.Font.Style+[fsUnderline];
    if not HotTrack or not(htUnderlineCold in HotTrackStyles) then
      Canvas.Font.Style:=Font.Style;
    if (cdsHot in State) and HotTrack then begin
      if htUnderlineHot in HotTrackStyles then
        Canvas.Font.Style:=Canvas.Font.Style+[fsUnderline];
      Canvas.Font.Color:=clHotLight;
    end;
    if Item.ImageIndex=-2 then
      Canvas.Font.Style:=[fsBold];
  end;
end;

procedure TwndMain.NetworkChangeNotifierChange(Sender: TObject; ARes: Cardinal);
begin
  DisplayData;
end;

procedure TwndMain.DisplayData;
var
  i: Integer;
begin
  Timer.Enabled:=False;
  List.Items.BeginUpdate;
  try
    List.Items.Clear;
    for i:=0 to FNC.AdapterCount-1 do
      with List.Items.Add do begin
        Caption:=FNC.Adapter[i].Name;
        SubItems.Add('');
        SubItems.Add('');
        SubItems.Add(GetNameCommaText(FNC.Adapter[i].IPv4Address));
        SubItems.Add(FNC.Adapter[i].IPv6Address.CommaText);
        SubItems.Add(GetValueCommaText(FNC.Adapter[i].IPv4Address));
        SubItems.Add(FNC.Adapter[i].MACAddress);
        SubItems.Add(AdapterTypes[FNC.Adapter[i].Typ]);
        SubItems.Add(AdapterOperationalStatus[FNC.Adapter[i].OperStatus]);
        SubItems.Add(AdapterAdminStatus[FNC.Adapter[i].AdminStatus]);
        SubItems.Add(IntToStr(FNC.Adapter[i].MaxSpeed div 1000000));
        SubItems.Add(IntToStr(FNC.Adapter[i].MTU));
        if FNC.BestInterfaceIdx=FNC.Adapter[i].IntfIdx then
          ImageIndex:=-2
        else
          ImageIndex:=-1;
      end;
  finally
    List.Items.EndUpdate;
    Timer.Enabled:=True;
  end;
end;

procedure TwndMain.TimerTimer(Sender: TObject);
var
  i: integer;
  inb,outb: UInt64;
begin
  List.Items.BeginUpdate;
  try
    for i:=0 to FNC.AdapterCount-1 do
      if FNC.Adapter[i].OperStatus>=aosConnected then begin
        FNC.GetTraffic(FNC.Adapter[i].IntfIdx,inb,outb);
        if inb>0 then
          List.Items[i].SubItems[0]:=NormalizeFloatDataValue(inb/1)
        else
          List.Items[i].SubItems[0]:='';
        if outb>0 then
          List.Items[i].SubItems[1]:=NormalizeFloatDataValue(outb/1)
        else
          List.Items[i].SubItems[1]:='';
      end;
  finally
    List.Items.EndUpdate;
  end;
end;

end.
