unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, MSI_SMBIOS;

type
  TwndMain = class(TForm)
    Memo: TMemo;
    Label1: TLabel;
    bAPI: TButton;
    bLoad: TButton;
    od: TOpenDialog;
    bSave: TButton;
    sd: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure bSMBIOSClick(Sender: TObject);
    procedure bLoadClick(Sender: TObject);
    procedure bSaveClick(Sender: TObject);
  private
    FSMBIOS: TMiTeC_SMBIOS;
  public
    procedure DisplayData;
  end;

var
  wndMain: TwndMain;

implementation

uses MiTeC_StrUtils;

{$R *.dfm}

function GetMachine :string;
var
  n :Cardinal;
  buf :PChar;
const
  rkMachine = {HKEY_LOCAL_MACHINE}'\SYSTEM\CurrentControlSet\Control\ComputerName\ComputerName';
    rvMachine = 'ComputerName';
begin
  n:=255;
  buf:=stralloc(n);
  GetComputerName(buf,n);
  result:=buf;
  strdispose(buf);
end;

procedure TwndMain.bLoadClick(Sender: TObject);
begin
  if not od.Execute then
    Exit;
  od.InitialDir:=ExtractFilePath(od.Filename);
  sd.InitialDir:=od.InitialDir;
  Screen.Cursor:=crHourGlass;
  try
    FSMBIOS.LoadFromFile(od.Filename);
    DisplayData;
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure TwndMain.bSaveClick(Sender: TObject);
begin
  if FSMBIOS.Reader.Size=0 then
    Exit;
  sd.Filename:=Format('%s_%d%d%d.smbios',[GetMachine,FSMBIOS.Reader.MajorVersion,FSMBIOS.Reader.MinorVersion,FSMBIOS.Reader.DMIRevision]);
  sd.Filename:=StringReplace(sd.Filename,',',' ',[rfReplaceAll]);
  sd.Filename:=StringReplace(sd.Filename,'/','-',[rfReplaceAll]);

  if not sd.Execute then
    Exit;

  sd.InitialDir:=ExtractFilePath(sd.Filename);
  od.InitialDir:=sd.InitialDir;
  FSMBIOS.Reader.SaveToFile(sd.Filename);
end;

procedure TwndMain.bSMBIOSClick(Sender: TObject);
begin
  Screen.Cursor:=crHourGlass;
  try
    FSMBIOS.RefreshData;
    DisplayData;
  finally
    Screen.Cursor:=crDefault;
  end;
end;

procedure TwndMain.DisplayData;
var
  i: integer;
  s: string;
begin
  Memo.Lines.BeginUpdate;
  try
    Memo.Lines.Clear;
    Memo.Lines.Add(Format('SMBIOS Version: %s',[FSMBIOS.VersionString]));
    Memo.Lines.Add(Format('Length: %d',[FSMBIOS.Reader.Size]));

    Memo.Lines.Add(Format('System: %s',[FSMBIOS.SystemString]));
    Memo.Lines.Add(Format('BIOS: %s',[Trim(FSMBIOS.BIOSVendor+' '+FSMBIOS.BIOSVersion+' '+FSMBIOS.BIOSDate)]));
    Memo.Lines.Add(Format('Baseboard: %s',[Trim(FSMBIOS.MainBoardModel+' '+FSMBIOS.MainBoardVersion)]));
    for i:=0 to FSMBIOS.TPMDeviceCount-1 do
      Memo.Lines.Add(Format('TPM: %s (%d.%d)',[Trim(Trim(FSMBIOS.TPMDevice[i].VendorID)+' '+FSMBIOS.TPMDevice[i].Description),FSMBIOS.TPMDevice[i].MajorSpecVersion,FSMBIOS.TPMDevice[i].MinorSpecVersion]));
    if FSMBIOS.ProcessorCount>0 then begin
      if Trim(FSMBIOS.Processor[0].Version)='' then
        s:=FSMBIOS.Processor[0].Manufacturer
      else
        s:=Trim(FSMBIOS.Processor[0].Version);
      s:=Format('CPU: %d x %s',[FSMBIOS.ProcessorCount,s]);
      s:=Format('%s - %d MHz',[s,FSMBIOS.Processor[0].Frequency]);
      if FSMBIOS.Processor[0].CoreCount>0 then
        s:=Format('%s (%d cores, %d threads)',[s,FSMBIOS.Processor[0].CoreCount,FSMBIOS.Processor[0].ThreadCount]);
      Memo.Lines.Add(s);
    end;
    Memo.Lines.Add(Format('Memory: %s %s',[NormalizeDataValue(FSMBIOS.InstalledMemory),FSMBIOS.MemoryType]));
    Memo.Lines.Add(Format('Enclosure: %s',[ChassisTypes[FSMBIOS.ChassisModel]]));
  finally
    Memo.Lines.EndUpdate;
  end;
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  FSMBIOS:=TMiTeC_SMBIOS.Create(nil);
  sd.InitialDir:=ExtractFilePath(Application.EXEName);
  od.InitialDir:=sd.InitialDir;
end;

end.

