unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, MSI_Common, MSI_SystemInfo;

const
  WM_REFRESH = WM_USER+1;

type
  TwndMain = class(TForm)
    lCPU: TLabel;
    cbxCPUFreq: TCheckBox;
    cbxCPUCore: TCheckBox;
    cbxCPU64: TCheckBox;
    lRAM: TLabel;
    cbxRAMSize: TCheckBox;
    lStor: TLabel;
    cbxStorSize: TCheckBox;
    lSF: TLabel;
    cbxSFUEFI: TCheckBox;
    cbxSFSB: TCheckBox;
    lTPM: TLabel;
    cbxTPMVer: TCheckBox;
    lGC: TLabel;
    cbxGC: TCheckBox;
    lDsip: TLabel;
    cbxDispDiag: TCheckBox;
    cbxDispColor: TCheckBox;
    Bevel1: TBevel;
    lResult: TLabel;
    imgOK: TImage;
    imgFail: TImage;
    procedure cbxClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    SI: TMiTeC_SystemInfo;
    sl: TStringList;
  protected
    procedure WMREFRESH(var AMsg: TMessage); message WM_REFRESH;
  public
  end;

var
  wndMain: TwndMain;

implementation

uses MiTeC_Routines, MiTeC_LSAAPI, MSI_Defs, MiTeC_Windows, MiTeC_Firmware, MSI_CPU;

{$R *.dfm}

procedure TwndMain.cbxClick(Sender: TObject);
begin
  TCheckBox(Sender).OnClick:=nil;
  TCheckBox(Sender).Checked:=not TCheckBox(Sender).Checked;
  TCheckBox(Sender).OnClick:=cbxClick;
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  SI:=TMiTeC_SystemInfo.Create(Self);
  EnablePrivilege(SE_SYSTEM_ENVIRONMENT_NAME);
  sl:=TStringList.Create;
end;

procedure TwndMain.FormShow(Sender: TObject);
begin
  PostMessage(Handle,WM_REFRESH,0,0);
end;

procedure TwndMain.WMREFRESH(var AMsg: TMessage);
var
  i,p: integer;
  ok: boolean;
  disp: boolean;
  s: string;
  rs: TResourceStream;
begin
  Screen.Cursor:=crHourglass;
  try
    Update;
    if (OSVIX.wProductType=VER_NT_WORKSTATION) then begin

      SI.OS.DetectUpdatesAndHotfixes:=False;
      SI.RefreshData(soOverview);

      s:='';
      if SI.CPU.Vendor=cvIntel then
        s:='INTEL'
      else if SI.CPU.Vendor=cvAMD then
        s:='AMD';

      if s<>'' then begin
        rs:=TResourceStream.Create(hInstance,s,RT_RCDATA);
        try
          sl.LoadFromStream(rs);
        finally
          rs.Free;
        end;
      end;

      s:=Uppercase(SI.CPU.CPUName);
      p:=-1;
      for i:=0 to sl.Count-1 do
        if Pos(Uppercase(sl[i]),s)>0 then begin
          p:=i;
          Break;
        end;

      cbxCPUFreq.Checked:=(SI.CPU.Frequency>=1000);
      cbxCPUCore.Checked:=(SI.CPU.CoreCount>=2);
      cbxCPU64.Checked:=(SI.CPU.Architecture in [PROCESSOR_ARCHITECTURE_IA32_ON_WIN64,PROCESSOR_ARCHITECTURE_AMD64]) and ((sl.Count=0) or (p>-1));

      cbxRAMSize.Checked:=(SI.Machine.SMBIOS.InstalledMemory>=4294967296);

      SI.Disk.Drive:=SI.Disk.SystemDisk;
      cbxStorSize.Checked:=(SI.Disk.Capacity>=68719476736);

      cbxSFUEFI.Checked:=SI.Machine.BIOS.UEFI;
      cbxSFSB.Checked:=SI.Machine.BIOS.SecureBoot in [uefisbDisabled,uefisbEnabled];

      cbxTPMVer.Checked:=(SI.Machine.TPM.InterfaceVersion<>'') and (StrToIntDef(SI.Machine.TPM.InterfaceVersion[1],0)>=2);

      for i:=0 to SI.Display.AdapterCount-1 do
        if (SI.Display.Adapter[i].WDDMVersion<>'') and (StrToIntDef(SI.Display.Adapter[i].WDDMVersion[1],0)>=2) and (SI.Display.Adapter[i].DXVersion>=12) then begin
          cbxGC.Checked:=True;
          Break;
        end;

      disp:=False;
      for i:=0 to SI.Monitor.Count-1 do
        if (SI.Monitor.Monitors[i].EDIDRecord.Diagonal>=9) then begin
          cbxDispDiag.Checked:=(SI.Monitor.Monitors[i].EDIDRecord.MaxVRes>=720) and (SI.Monitor.Monitors[i].EDIDRecord.MaxHRes>=1280);
          cbxDispColor.Checked:=SI.Monitor.Monitors[i].ColorDepth>=8;
          Break;
        end;

      ok:=True;
      for i:=0 to ControlCount-1 do
        if Controls[i] is TCheckBox then begin
          ok:=ok and TCheckBox(Controls[i]).Checked;
          if not TCheckBox(Controls[i]).Checked then
            TCheckBox(Controls[i]).Font.Style:=[fsStrikeOut];
        end;
      if ok then begin
        lResult.Caption:='Test PASSED';
        imgOK.Show;
      end else begin
        lResult.Caption:='Test FAILED';
        imgFail.Show;
      end;

      if (TrueOS>=oswin11) then begin
        lResult.Caption:=Format('%s already installed',[TrueWindowsName]);
        imgOK.Show;
      end;
    end;

    for i:=0 to ControlCount-1 do
      if Controls[i] is TCheckBox then
        TCheckBox(Controls[i]).OnClick:=cbxClick;
  finally
    Screen.Cursor:=crDefault;
  end;
end;

end.
